@@ -33,6 +33,7 @@ Node *newNode()
     n->retrieved = 0;
     n->cancelled = 0;
     n->played = 0;
+    n->frombutton = 0;
     n->bytes = 0;
     n->totalbytes = 0;
     n->cachebytes = 0;
@@ -259,7 +260,8 @@ void addToList(nsPluginInstance * instance, char *item, Node * parent,
 	}
 	addToEnd(parent, node);
 	parent->play = 0;
-	if (isMms(item) || strstr(node->url, parent->url) != NULL) {
+	if (isMms(item, instance->nomediacache)
+	    || strstr(node->url, parent->url) != NULL) {
 	    node->mmsstream = 1;
 	} else {
 	    filename = getURLFilename(localitem);
@@ -287,7 +289,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
     char buffer[16 * 1024];
     char buffer_lower[16 * 1024];
     char url[1024];
-    unsigned long int size, found, i, remainder;
+    unsigned long int size, found, i;
+    long int remainder;
     char *p;
     char *nextrmda = NULL;
     char *rdrf;
@@ -414,12 +417,14 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		    remainder = size - ((long) p - (long) buffer_lower);
 		    if (remainder < 1)
 			break;
+
 		    // seq tags are optional, but if we do have
 		    // one see if it has a speed and if so, find it
 		    seqtag = (char *) memmem(p, remainder, "<seq", 4);
 		    if (seqtag != NULL) {
 			if (DEBUG)
 			    printf("[smil] found seq tag\n");
+
 			remainder =
 			    size - ((long) seqtag - (long) buffer_lower);
 			rate =
@@ -434,8 +439,10 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 				printf("[smil] found bit rate %i\n",
 				       speed);
 			} else {
-			    // if we don't fine a speed, then set the speed to -1
+			    // if we don't find a speed, then set the speed to -1
 			    speed = -1;
+			    // and reset p so that we can find the video tag
+			    seqtag = (char *) p;
 			}
 		    } else {
 			// if we didn't find a seqtag, then set seqtag equal to p
@@ -582,19 +589,22 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		    if (p == NULL)
 			break;
 		    if ((p - 9) > buffer_lower) {
-			if (memmem(p - 5, i, "logo", 4) != NULL) {
+			// look backwards in the file and see if the href is preceeded by
+			// a qualifier tag, but don't look too far forward otherwise we may
+			// find another tag of this type
+			if (memmem(p - 5, 5, "logo", 4) != NULL) {
 			    if (DEBUG > 1)
 				printf("found logo in ASX\n");
 			    p = p + 4;
 			    continue;
 			}
-			if (memmem(p - 7, i, "banner", 6) != NULL) {
+			if (memmem(p - 7, 7, "banner", 6) != NULL) {
 			    if (DEBUG > 1)
 				printf("found banner in ASX\n");
 			    p = p + 4;
 			    continue;
 			}
-			if (memmem(p - 9, i, "moreinfo", 8) != NULL) {
+			if (memmem(p - 9, 9, "moreinfo", 8) != NULL) {
 			    if (DEBUG > 1)
 				printf("found moreinfo in ASX\n");
 			    p = p + 4;
