@@ -1,4 +1,5 @@
 #include "plugin.h"
+#include <sys/stat.h>
 
 #ifndef STATICDECLS
 #define STATICDECLS
@@ -44,6 +45,12 @@ char *GetMIMEDescription()
     int enable_ogg;
     int enable_smil;
     int enable_helix;
+    int enable_wmp;
+    int enable_qt;
+    int enable_rm;
+    int enable_gmp;
+    int enable_mp3;
+
 #ifdef STD
     FILE *customtypes;
     char customline[255];
@@ -67,6 +74,10 @@ char *GetMIMEDescription()
     enable_ogg = 1;
     enable_smil = 1;
     enable_helix = 1;
+    enable_wmp = 1;
+    enable_qt = 1;
+    enable_rm = 1;
+    enable_gmp = 1;
     use_custom_mime_types = 0;
 
     DEBUG = 0;			//set to 1 when debugging this function
@@ -118,7 +129,7 @@ char *GetMIMEDescription()
 		    continue;
 		}
 		// Ogg Vorbis
-		if (strncasecmp(buffer, "enable-ogg", 11) == 0) {
+		if (strncasecmp(buffer, "enable-ogg", 10) == 0) {
 		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
 		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
 		    sscanf(parse, "%i", &enable_ogg);
@@ -136,6 +147,51 @@ char *GetMIMEDescription()
 			       use_custom_mime_types);
 		    continue;
 		}
+		// Windows Media Player
+		if (strncasecmp(buffer, "enable-wmp", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_wmp);
+		    if (DEBUG)
+			printf("wmp:%i\n", enable_wmp);
+		    continue;
+		}
+		// QuickTime
+		if (strncasecmp(buffer, "enable-qt", 9) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_qt);
+		    if (DEBUG)
+			printf("qt:%i\n", enable_qt);
+		    continue;
+		}
+		// RealMedia
+		if (strncasecmp(buffer, "enable-rm", 9) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_rm);
+		    if (DEBUG)
+			printf("rm:%i\n", enable_rm);
+		    continue;
+		}
+		// Google Media Player
+		if (strncasecmp(buffer, "enable-gmp", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_gmp);
+		    if (DEBUG)
+			printf("gmp:%i\n", enable_gmp);
+		    continue;
+		}
+		// MP3
+		if (strncasecmp(buffer, "enable-mp3", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_mp3);
+		    if (DEBUG)
+			printf("mp3:%i\n", enable_mp3);
+		    continue;
+		}
 
 	    }
 	    fclose(config);
@@ -183,54 +239,64 @@ char *GetMIMEDescription()
     } else {
 
 #ifdef QT
-	strlcat(MimeTypes,
-		"video/quicktime:mov:Quicktime;"
-		"video/x-quicktime:mov:Quicktime;"
-		"image/x-quicktime:mov:Quicktime;"
-		"video/quicktime:mp4:Quicktime;"
-		"video/quicktime:sdp:Quicktime - Session Description Protocol;"
-		"application/x-quicktimeplayer:mov:Quicktime;",
-		sizeof(MimeTypes));
-
-	if (enable_smil)
+	if (enable_qt) {
 	    strlcat(MimeTypes,
-		    "application/smil:smil:SMIL;", sizeof(MimeTypes));
+		    "video/quicktime:mov:Quicktime;"
+		    "video/x-quicktime:mov:Quicktime;"
+		    "image/x-quicktime:mov:Quicktime;"
+		    "video/quicktime:mp4:Quicktime;"
+		    "video/quicktime:sdp:Quicktime - Session Description Protocol;"
+		    "application/x-quicktimeplayer:mov:Quicktime;",
+		    sizeof(MimeTypes));
 
+	    if (enable_smil)
+		strlcat(MimeTypes,
+			"application/smil:smil:SMIL;", sizeof(MimeTypes));
+	}
 #endif
 
 #ifdef WMP
-	strlcat(MimeTypes,
-		"application/asx:*:Media Files;"
-		"video/x-ms-asf-plugin:*:Media Files;"
-		"video/x-msvideo:avi,*:AVI;"
-		"video/msvideo:avi,*:AVI;"
-		"application/x-mplayer2:*:Media Files;"
-		"application/x-ms-wmv:wmv,*:Microsoft WMV video;"
-		"video/x-ms-asf:asf,asx,*:Media Files;"
-		"video/x-ms-wm:wm,*:Media Files;"
-		"video/x-ms-wmv:wmv,*:Microsoft WMV video;"
-		"video/x-ms-wmp:wmp,*:Windows Media;"
-		"video/x-ms-wvx:wvx,*:Windows Media;"
-		"audio/x-ms-wax:wax,*:Windows Media;"
-		"audio/x-ms-wma:wma,*:Windows Media;"
-		"application/x-drm-v2:asx,*:Windows Media;"
-		"audio/wav:wav,*:Microsoft wave file;"
-		"audio/x-wav:wav,*:Microsoft wave file;",
-		sizeof(MimeTypes));
+	if (enable_wmp) {
+	    strlcat(MimeTypes,
+		    "application/asx:*:Media Files;"
+		    "video/x-ms-asf-plugin:*:Media Files;"
+		    "video/x-msvideo:avi,*:AVI;"
+		    "video/msvideo:avi,*:AVI;"
+		    "application/x-mplayer2:*:Media Files;"
+		    "application/x-ms-wmv:wmv,*:Microsoft WMV video;"
+		    "video/x-ms-asf:asf,asx,*:Media Files;"
+		    "video/x-ms-wm:wm,*:Media Files;"
+		    "video/x-ms-wmv:wmv,*:Microsoft WMV video;"
+		    "audio/x-ms-wmv:wmv,*:Windows Media;"
+		    "video/x-ms-wmp:wmp,*:Windows Media;"
+		    "video/x-ms-wvx:wvx,*:Windows Media;"
+		    "audio/x-ms-wax:wax,*:Windows Media;"
+		    "audio/x-ms-wma:wma,*:Windows Media;"
+		    "application/x-drm-v2:asx,*:Windows Media;"
+		    "audio/wav:wav,*:Microsoft wave file;"
+		    "audio/x-wav:wav,*:Microsoft wave file;",
+		    sizeof(MimeTypes));
+	}
 #endif
 
 #ifdef RM
-	strlcat(MimeTypes,
-		"audio/x-pn-realaudio:ram,rm:RealAudio;"
-		"audio/x-realaudio:ra:RealAudio;", sizeof(MimeTypes));
-
-	if (enable_helix)
+	if (enable_rm) {
 	    strlcat(MimeTypes,
-		    "audio/x-pn-realaudio-plugin:rpm:RealAudio;", sizeof(MimeTypes));
+		    "audio/x-pn-realaudio:ram,rm:RealAudio;"
+		    "application/vnd.rn-realmedia:rm:RealMedia;"
+		    "application/vnd.rn-realaudio:ra,ram:RealAudio;"
+		    "video/vnd.rn-realvideo:rv:RealVideo;"	
+		    "audio/x-realaudio:ra:RealAudio;", sizeof(MimeTypes));
 
-	if (enable_smil)
-	    strlcat(MimeTypes,
-		    "application/smil:smil:SMIL;", sizeof(MimeTypes));
+	    if (enable_helix)
+		strlcat(MimeTypes,
+			"audio/x-pn-realaudio-plugin:rpm:RealAudio;",
+			sizeof(MimeTypes));
+
+	    if (enable_smil)
+		strlcat(MimeTypes,
+			"application/smil:smil:SMIL;", sizeof(MimeTypes));
+	}
 #endif
 
 #ifdef STD
@@ -244,10 +310,14 @@ char *GetMIMEDescription()
 		    "audio/x-mpeg:mpg,mpeg:MPEG;"
 		    "audio/mpeg2:mp2:MPEG audio;"
 		    "audio/x-mpeg2:mp2:MPEG audio;"
-		    "audio/mpeg3:mp3:MPEG audio;"
-		    "audio/x-mpeg3:mp3:MPEG audio;"
-		    "audio/mp3:mp3:MPEG audio;"
 		    "video/mp4:mp4:MPEG 4 Video;", sizeof(MimeTypes));
+
+	    if (enable_mp3)
+		strlcat(MimeTypes,
+			"audio/mpeg3:mp3:MPEG audio;"
+			"audio/x-mpeg3:mp3:MPEG audio;"
+			"audio/mp3:mp3:MPEG audio;", sizeof(MimeTypes));
+
 	}
 
 	if (enable_ogg) {
@@ -270,12 +340,22 @@ char *GetMIMEDescription()
 	strlcat(MimeTypes,
 		"application/x-nsv-vp3-mp3:nsv:Nullsoft Streaming Video;",
 		sizeof(MimeTypes));
+	// DIVX
+	strlcat(MimeTypes,
+		"video/vnd.divx:divx:DivX Media Format;",
+		sizeof(MimeTypes));
+	// Basic
+	strlcat(MimeTypes,
+		"audio/basic:au,snd:Basic Audio File;"
+    		"audio/x-basic:au,snd:Basic Audio File;", sizeof(MimeTypes));
 #endif
 	// Google Video
 #ifdef GMP
-	strlcat(MimeTypes,
-		"application/x-google-vlc-plugin::Google Video;",
-		sizeof(MimeTypes));
+	if (enable_gmp) {
+	    strlcat(MimeTypes,
+		    "application/x-google-vlc-plugin::Google Video;",
+		    sizeof(MimeTypes));
+	}
 #endif
 
     }
@@ -362,6 +442,7 @@ NPError GetValue(NPPVariable variable, void *value)
 void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 {
     int i;
+    int real_master_console = 0;
     char parse[1000];
     char *cp;
 
@@ -372,11 +453,12 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
     instance->mInstance = parameters->instance;
     instance->mimetype = strdup(parameters->type);
 
+/*
     if ((strstr(instance->mimetype, "mpeg") != NULL)
 	|| (strstr(instance->mimetype, "mp3") != NULL)) {
 	instance->cache_percent = 100;
     }
-
+*/
     if (parameters->mode == NP_EMBED) {
 
 	if (DEBUG)
@@ -388,6 +470,26 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		printf("Argument Value: %s\n", parameters->argv[i]);
 	    }
 
+	    if (strncasecmp(parameters->argn[i], "debug", 5) == 0) {
+		lowercase(parameters->argv[i]);
+
+		if (strstr(parameters->argv[i], "true")
+		    || strstr(parameters->argv[i], "yes")
+		    || strstr(parameters->argv[i], "1")) {
+		    DEBUG = 1;
+		}
+	    }
+
+	    if (strncasecmp(parameters->argn[i], "nocache", 7) == 0) {
+		lowercase(parameters->argv[i]);
+
+		if (strstr(parameters->argv[i], "true")
+		    || strstr(parameters->argv[i], "yes")
+		    || strstr(parameters->argv[i], "1")) {
+		    instance->nomediacache = 1;
+		}
+	    }
+
 	    if (strncasecmp(parameters->argn[i], "src", 3) == 0) {
 		if (instance->fname != NULL) {
 		    if (strcmp(instance->fname, parameters->argv[i]) != 0) {
@@ -497,7 +599,13 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 	    if (strncasecmp(parameters->argn[i], "target", 6) == 0) {
 		if (strncasecmp(parameters->argv[i], "quicktimeplayer", 15)
 		    == 0) {
+#ifdef GTK2_ENABLED
+		    instance->targetplayer = 1;
+#endif
+#ifndef GTK2_ENABLED
+		    instance->targetplayer = 0;
 		    instance->noembed = 1;
+#endif
 		}
 #ifdef GMP
 		instance->fname = strdup(parameters->argv[i]);
@@ -548,7 +656,7 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		if (strstr(parameters->argv[i], "true")
 		    || strstr(parameters->argv[i], "yes")
 		    || strstr(parameters->argv[i], "1")) {
-		    instance->autostart = 1;
+		    // instance->autostart = 1;
 		} else {
 		    instance->autostart = 0;
 		}
@@ -657,17 +765,19 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    || strstr(parameters->argv[i], "1")
 		    || strstr(parameters->argv[i], "all")
 		    || strstr(parameters->argv[i], "statusfield")
-		    || strstr(parameters->argv[i], "statusbar")
 		    || strstr(parameters->argv[i], "statuspanel")
 		    || strstr(parameters->argv[i], "playbutton")
 		    || strstr(parameters->argv[i], "volumeslider")
 		    || strstr(parameters->argv[i], "stopbutton")
-		    || strstr(parameters->argv[i], "positionslider")
-		    || strstr(parameters->argv[i], "controlpanel")) {
+		    || strstr(parameters->argv[i], "positionslider")) {
 		    instance->controlwindow = 1;
 		} else {
 		    instance->controlwindow = 0;
 		}
+	        if (strstr(parameters->argv[i], "controlpanel") 
+		    || strstr(parameters->argv[i], "statusbar")) {
+			instance->showcontrols = 1;
+		}
 		if (strstr(parameters->argv[i], "imagewindow")) {
 		    instance->controlwindow = 0;
 		}
@@ -675,6 +785,21 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    printf("controlwindow=%i\n", instance->controlwindow);
 	    }
 
+	    if ((strncasecmp(parameters->argn[i], "console", 7) == 0)
+		&& (strstr(instance->mimetype, "quicktime") == NULL)) {
+
+		lowercase(parameters->argv[i]);
+		if (DEBUG)
+		    printf("argv[i]=%s\n", parameters->argv[i]);
+		
+		if (strstr(parameters->argv[i], "_master")) {
+			real_master_console = 1;
+		} else {
+			real_master_console = 0;
+		}
+
+	    }		
+	
 	    if ((strncasecmp
 		 (parameters->argn[i], "onmediacomplete", 15) == 0)
 		|| (strncasecmp(parameters->argn[i], "onendofstream", 13)
@@ -697,6 +822,122 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 			   instance->mediaCompleteCallback);
 	    }
 
+	    if (strncasecmp
+		(parameters->argn[i], "onmediacompletewitherror",
+		 24) == 0) {
+		instance->mediaCompleteWithErrorCallback =
+		    (char *) NPN_MemAlloc(strlen(parameters->argv[i]) +
+					  12);
+		if (strncasecmp(parameters->argv[i], "javascript:", 11)
+		    == 0) {
+		    snprintf(instance->mediaCompleteWithErrorCallback,
+			     strlen(parameters->argv[i]), "%s",
+			     parameters->argv[i]);
+		} else {
+		    snprintf(instance->mediaCompleteWithErrorCallback,
+			     strlen(parameters->argv[i]) + 12,
+			     "javascript:%s", parameters->argv[i]);
+		}
+		if (DEBUG)
+		    printf("mediaCompleteWithErrorCallback=%s\n",
+			   instance->mediaCompleteWithErrorCallback);
+	    }
+
+	    if (strncasecmp(parameters->argn[i], "onclick", 7) == 0) {
+		instance->mouseClickCallback =
+		    (char *) NPN_MemAlloc(strlen(parameters->argv[i]) +
+					  12);
+		if (strncasecmp(parameters->argv[i], "javascript:", 11)
+		    == 0) {
+		    snprintf(instance->mouseClickCallback,
+			     strlen(parameters->argv[i]), "%s",
+			     parameters->argv[i]);
+		} else {
+		    snprintf(instance->mouseClickCallback,
+			     strlen(parameters->argv[i]) + 12,
+			     "javascript:%s", parameters->argv[i]);
+		}
+		if (DEBUG)
+		    printf("mouseClickCallback=%s\n",
+			   instance->mouseClickCallback);
+	    }
+
+	    if (strncasecmp(parameters->argn[i], "onmousedown", 11) == 0) {
+		instance->mouseDownCallback =
+		    (char *) NPN_MemAlloc(strlen(parameters->argv[i]) +
+					  12);
+		if (strncasecmp(parameters->argv[i], "javascript:", 11)
+		    == 0) {
+		    snprintf(instance->mouseDownCallback,
+			     strlen(parameters->argv[i]), "%s",
+			     parameters->argv[i]);
+		} else {
+		    snprintf(instance->mouseDownCallback,
+			     strlen(parameters->argv[i]) + 12,
+			     "javascript:%s", parameters->argv[i]);
+		}
+		if (DEBUG)
+		    printf("mouseDownCallback=%s\n",
+			   instance->mouseDownCallback);
+	    }
+
+	    if (strncasecmp(parameters->argn[i], "onmouseup", 9) == 0) {
+		instance->mouseUpCallback =
+		    (char *) NPN_MemAlloc(strlen(parameters->argv[i]) +
+					  12);
+		if (strncasecmp(parameters->argv[i], "javascript:", 11)
+		    == 0) {
+		    snprintf(instance->mouseUpCallback,
+			     strlen(parameters->argv[i]), "%s",
+			     parameters->argv[i]);
+		} else {
+		    snprintf(instance->mouseUpCallback,
+			     strlen(parameters->argv[i]) + 12,
+			     "javascript:%s", parameters->argv[i]);
+		}
+		if (DEBUG)
+		    printf("mouseUpCallback=%s\n",
+			   instance->mouseUpCallback);
+	    }
+
+	    if (strncasecmp(parameters->argn[i], "onmouseover", 11) == 0) {
+		instance->mouseEnterCallback =
+		    (char *) NPN_MemAlloc(strlen(parameters->argv[i]) +
+					  12);
+		if (strncasecmp(parameters->argv[i], "javascript:", 11)
+		    == 0) {
+		    snprintf(instance->mouseEnterCallback,
+			     strlen(parameters->argv[i]), "%s",
+			     parameters->argv[i]);
+		} else {
+		    snprintf(instance->mouseEnterCallback,
+			     strlen(parameters->argv[i]) + 12,
+			     "javascript:%s", parameters->argv[i]);
+		}
+		if (DEBUG)
+		    printf("mouseEnterCallback=%s\n",
+			   instance->mouseEnterCallback);
+	    }
+
+	    if (strncasecmp(parameters->argn[i], "onmouseout", 10) == 0) {
+		instance->mouseLeaveCallback =
+		    (char *) NPN_MemAlloc(strlen(parameters->argv[i]) +
+					  12);
+		if (strncasecmp(parameters->argv[i], "javascript:", 11)
+		    == 0) {
+		    snprintf(instance->mouseLeaveCallback,
+			     strlen(parameters->argv[i]), "%s",
+			     parameters->argv[i]);
+		} else {
+		    snprintf(instance->mouseLeaveCallback,
+			     strlen(parameters->argv[i]) + 12,
+			     "javascript:%s", parameters->argv[i]);
+		}
+		if (DEBUG)
+		    printf("mouseLeaveCallback=%s\n",
+			   instance->mouseLeaveCallback);
+	    }
+
 	    if (strncasecmp
 		(parameters->argn[i], "onvisible",
 		 strlen("onvisible")) == 0) {
@@ -791,9 +1032,12 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		}
 	    }
 	}
-
+	if (instance->controlwindow == 1 && real_master_console == 1)
+	    instance->controlwindow = 0;
+    
 	if (instance->controlwindow == 0) {
-	    if ((instance->fname != NULL) && (!isMms(instance->fname))) {
+	    if ((instance->fname != NULL)
+		&& (!isMms(instance->fname, instance->nomediacache))) {
 		NPN_GetURL(parameters->instance, instance->fname, NULL);
 	    }
 	}
@@ -856,6 +1100,33 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    continue;
 		}
 
+		if (strncasecmp(buffer, "showlogo", 8) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->showlogo);
+		    if (instance->showlogo != 0)
+			instance->showlogo = 1;
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "showtime", 8) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->showtime);
+		    if (instance->showtime != 0)
+			instance->showtime = 1;
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "hidestatus", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->hidestatus);
+		    if (instance->hidestatus != 0)
+			instance->hidestatus = 1;
+		    continue;
+		}
+
 		if (strncasecmp(buffer, "novop", 5) == 0) {
 		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
 		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
@@ -894,6 +1165,15 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    continue;
 		}
 
+		if (strncasecmp(buffer, "af", 2) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "\n"));
+		    if (instance->af != NULL)
+			free(instance->af);
+		    instance->af = strdup(parse);
+		    continue;
+		}
+
 		if (strncasecmp(buffer, "prefer-aspect", 13) == 0) {
 		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
 		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
@@ -967,6 +1247,7 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    if (instance->download_dir != NULL)
 			free(instance->download_dir);
 		    instance->download_dir = strdup(parse);
+		    mkdir(instance->download_dir, 0777);
 		    continue;
 		}
 
@@ -1092,6 +1373,80 @@ void LoadConfigFile(nsPluginInstance * instance)
 			instance->useragent = strdup(parse);
 		    continue;
 		}
+		// SMIL
+		if (strncasecmp(buffer, "enable-smil", 11) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_smil);
+		    if (DEBUG)
+			printf("smil:%i\n", instance->enable_smil);
+		    continue;
+		}
+		// Helix RPM MimeType
+		if (strncasecmp(buffer, "enable-helix", 12) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_helix);
+		    if (DEBUG)
+			printf("helix:%i\n", instance->enable_helix);
+		    continue;
+		}
+		// Windows Media Player
+		if (strncasecmp(buffer, "enable-wmp", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_wmp);
+		    if (DEBUG)
+			printf("wmp:%i\n", instance->enable_wmp);
+		    continue;
+		}
+		// QuickTime
+		if (strncasecmp(buffer, "enable-qt", 9) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_qt);
+		    if (DEBUG)
+			printf("qt:%i\n", instance->enable_qt);
+		    continue;
+		}
+		// RealMedia
+		if (strncasecmp(buffer, "enable-rm", 9) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_rm);
+		    if (DEBUG)
+			printf("rm:%i\n", instance->enable_rm);
+		    continue;
+		}
+		// Google Media Player
+		if (strncasecmp(buffer, "enable-gmp", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_gmp);
+		    if (DEBUG)
+			printf("gmp:%i\n", instance->enable_gmp);
+		    continue;
+		}
+		// MP3
+		if (strncasecmp(buffer, "enable-mp3", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_mp3);
+		    if (DEBUG)
+			printf("mp3:%i\n", instance->enable_mp3);
+		    continue;
+		}
+		// Ogg
+		if (strncasecmp(buffer, "enable-ogg", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_ogg);
+		    if (DEBUG)
+			printf("ogg:%i\n", instance->enable_ogg);
+		    continue;
+		}
+
+
 
 	    }
 	    fclose(config);
