@@ -156,7 +156,7 @@ char *getURLBase(char *url)
     return base;
 }
 
-int isMms(char *url)
+int isMms(char *url, int nomediacache)
 {
     if (url == NULL)
 	return 0;
@@ -170,9 +170,25 @@ int isMms(char *url)
 	    printf("isMms = true\n");
 	return 1;
     } else {
-	if (DEBUG > 1)
-	    printf("isMms = false\nurl = %s\n", url);
-	return 0;
+	if (nomediacache) {
+	    if ((strncasecmp(url, "file://", 7) == 0)
+		|| (strncasecmp(url, "smb://", 6) == 0)
+		|| fexists(url)) {
+		if (DEBUG > 1)
+		    printf("isMms = false\nurl = %s\n", url);
+		return 0;
+
+	    } else {
+		if (DEBUG > 1)
+		    printf("isMms = true\n");
+		return 1;
+	    }
+
+	} else {
+	    if (DEBUG > 1)
+		printf("isMms = false\nurl = %s\n", url);
+	    return 0;
+	}
     }
 }
 
@@ -351,9 +367,9 @@ int URLcmp(const char *url1, const char *url2)
 			    if (tmp1hasq != tmp2hasq) {
 				ret = -1;
 			    } else {
-				if (strcmp(q1+1,q2+1) == 0) {
+				if (strcmp(q1 + 1, q2 + 1) == 0) {
 				    ret = 0;
-				} else { 
+				} else {
 				    ret = -1;
 				}
 			    }
@@ -470,16 +486,20 @@ void killmplayer(nsPluginInstance * instance)
     if (instance->paused == 1)
 	sendCommand(instance, "pause\n");
     sendCommand(instance, "quit\n");
+    pthread_mutex_lock(&(instance->read_mutex));
     instance->cancelled = 1;
+    pthread_mutex_unlock(&(instance->read_mutex));
 
-    count = 0;
-    while (instance->player != NULL && count < 1000) {
+/*    count = 0;
+    while (count < 500) {
+	if (DEBUG)
+	    printf("waiting for quit to be handled\n");
 	usleep(100);
 	count++;
-	if (DEBUG > 1)
-	    printf("waiting for quit to be handled\n");
+    	if (instance->player == NULL)
+	    break;
     }
-
+*/
     //we can do the following twice on the same thread with no ill effects
     pthread_cancel(instance->player_thread);
     pthread_join(instance->player_thread, &thread_return);
@@ -560,6 +580,12 @@ void killmplayer(nsPluginInstance * instance)
 	    instance->td->argv[count++] = NULL;
 	}
 	instance->threadsetup = 0;
+#ifdef GTK_ENABLED
+    	if (GTK_IS_WIDGET(instance->drawing_area)) {
+		gtk_widget_destroy(instance->drawing_area);
+		instance->drawing_area = NULL;
+	}
+#endif
     }
 }
 
@@ -576,7 +602,7 @@ void fullyQualifyURL(nsPluginInstance * instance, char *item,
 	printf("item: %s\nbaseurl: %s\nhostname: %s\n",
 	       item, instance->baseurl, instance->hostname);
 
-    if (!isMms(item)) {
+    if (!isMms(item, instance->nomediacache)) {
 	if ((strncasecmp(item, "http", 4) != 0)
 	    && (strncasecmp(item, "file", 4) != 0)) {
 
