@@ -7,8 +7,10 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <sys/param.h>
+#include <errno.h>
 
 extern int DEBUG;
+extern int errno;
 
 static void sig_child(int signo)
 {
@@ -63,6 +65,9 @@ gboolean mediacallback(void *data)
     if (instance->mediaCompleteCallback != NULL)
 	NPN_GetURL(instance->mInstance,
 		   instance->mediaCompleteCallback, "_self");
+    if (instance->mediaCompleteWithErrorCallback != NULL)
+	NPN_GetURL(instance->mInstance,
+		   instance->mediaCompleteWithErrorCallback, "_self");
     return FALSE;
 }
 #endif
@@ -103,11 +108,14 @@ void launchPlayerThread(nsPluginInstance * instance)
     if (DEBUG)
 	printf("In launchPlayerThread, state = %d\n", instance->state);
 
-    if (instance->threadlaunched == 1)
+    if (instance->threadlaunched == 1) {
+	if (DEBUG)
+	    printf("launchPlayerThread - joining thread\n");
 	pthread_join(instance->player_thread, &thread_return);
-
+    }
     if (instance->js_state == JS_STATE_UNDEFINED) {
-
+	if (DEBUG)
+	    printf("launchPlayerThread - creating new thread\n");
 	pthread_create(&(instance->player_thread),
 		       &(instance->thread_attr), playPlaylist,
 		       (void *) (instance->td));
@@ -148,17 +156,17 @@ void signalPlayerThread(nsPluginInstance * instance)
 	pthread_cond_signal(&(instance->playlist_complete_cond));
 	pthread_mutex_unlock(&(instance->playlist_cond_mutex));
 	instance->threadsignaled = 1;
-	pthread_mutex_unlock(&(instance->control_mutex));
     } else {
 	if (DEBUG)
 	    printf("****Player thread did not start correctly****\n");
-	pthread_mutex_unlock(&(instance->control_mutex));
     }
+    pthread_mutex_unlock(&(instance->control_mutex));
 }
 
-FILE *mypopen(char **argv, pid_t * pid, int *control)
+FILE *mypopen(char **argv, pid_t * pid, int *control,
+	      nsPluginInstance * instance)
 {
-    int filedesr[2], filedesw[2];
+    int filedesr[2], filedesw[2], err;
     pid_t child;
     long flags;
     sigset_t newmask;
@@ -189,8 +197,16 @@ FILE *mypopen(char **argv, pid_t * pid, int *control)
 	sigaddset(&newmask, SIGKILL);
 	pthread_sigmask(SIG_UNBLOCK, &newmask, 0L);
 	usleep(500);
-	if (execvp(argv[0], argv) < 0)
+	if (execvp(argv[0], argv) < 0) {
+	    err = errno;
+#ifdef GTK_ENABLED
+	    snprintf(instance->lastmessage, 1024, "Error: %i - %s", err,
+		     strerror(err));
+	    g_idle_add(gtkgui_message, instance);
+#endif
 	    perror("execv");
+
+	}
 	_exit(0);
 
     } else {
@@ -254,11 +270,14 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	instance->td->list = instance->list;
     }
 
+/*
     if (instance->href) {
 	if (DEBUG)
 	    printf("using href for url\n");
 	snprintf(instance->td->list->url, 1024, "%s", instance->href);
-    } else if (instance->fname) {
+    } else 
+*/
+    if (instance->fname) {
 	if (DEBUG)
 	    printf("using fname for url\n");
 	snprintf(instance->td->list->url, 1024, "%s", instance->fname);
@@ -268,12 +287,23 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	snprintf(instance->td->list->url, 1024, "%s", instance->url);
     }
 
+    if ((instance->fname == NULL) && (instance->url == NULL)) {
+	if (DEBUG)
+	    printf("using href for url\n");
+	snprintf(instance->td->list->url, 1024, "%s", instance->href);
+    }
+
     if (instance->mode == NP_FULL) {
 	snprintf(xval, 32, "%i", instance->window_width);
 	snprintf(yval, 32, "%i", instance->window_height);
     } else {
 	snprintf(xval, 32, "%i", instance->embed_width);
-	snprintf(yval, 32, "%i", instance->embed_height);
+	if (instance->maintain_aspect == 0 && instance->showcontrols == 1) {
+	    snprintf(yval, 32, "%i", instance->embed_height - 16);
+	} else {
+	    snprintf(yval, 32, "%i", instance->embed_height);
+	}
+
     }
 
     baseurl = NULL;
@@ -350,7 +380,6 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	}
     }
 
-
     if ((instance->mode == NP_EMBED) && (instance->noembed == 0)) {
 	if (instance->window != 0) {
 	    snprintf(buffer, 1024, "-wid");
@@ -370,10 +399,19 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			  instance->drawing_area, 0, 0);
 	    gtk_widget_show(instance->drawing_area);
+	    if (instance->targetplayer == 1)
+		gtk_widget_show(instance->gtkwidget);
 	    instance->player_window =
 		GDK_WINDOW_XWINDOW(instance->drawing_area->window);
+	    if (instance->targetplayer == 1)
+		gtk_widget_hide(instance->gtkwidget);
 	    snprintf(buffer, 1024, "0x%x", (int) instance->player_window);
 	    instance->td->argv[i++] = strdup(buffer);
+#ifdef GTK2_ENABLED
+	    g_signal_connect_after(G_OBJECT(instance->gtkwidget),
+				   "visibility-notify-event",
+				   G_CALLBACK(window_visible), instance);
+#endif
 #endif
 	} else {
 	    instance->player_window = 0;
@@ -428,7 +466,8 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 		}
 	    }
 #endif
-	    if (isRemoteDisplay == false) {
+	    if ((isRemoteDisplay == false)
+		&& (instance->targetplayer == 0)) {
 		if (instance->maintain_aspect == 1) {
 		    snprintf(buffer, 1024, "-xy");
 		    instance->td->argv[i++] = strdup(buffer);
@@ -448,10 +487,10 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	}
     }
 
-    if (instance->rtsp_use_tcp == 1) {
-	snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
-	instance->td->argv[i++] = strdup(buffer);
-    }
+//    if (instance->rtsp_use_tcp == 1) {
+//      snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
+//      instance->td->argv[i++] = strdup(buffer);
+//    }
 //    if (instance->cachesize > 0) {
 //      snprintf(buffer, 1024, "-cache");
 //      instance->td->argv[i++] = strdup(buffer);
@@ -472,7 +511,7 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	instance->td->argv[i++] = strdup(buffer);
 	snprintf(buffer, 1024, "%s", instance->vo);
 	instance->td->argv[i++] = strdup(buffer);
-	if ((strcmp(buffer, "x11") == 0)
+	if ((strncmp(buffer, "x11", 3) == 0)
 	    || (strstr(buffer, ",x11") != NULL)) {
 	    snprintf(buffer, 1024, "-zoom");	/* -vo x11 needs -zoom for rescaling */
 	    instance->td->argv[i++] = strdup(buffer);
@@ -486,6 +525,13 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	instance->td->argv[i++] = strdup(buffer);
     }
 
+    if (instance->af) {
+	snprintf(buffer, 1024, "-af");
+	instance->td->argv[i++] = strdup(buffer);
+	snprintf(buffer, 1024, "%s", instance->af);
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
     if (instance->output_display) {
 	snprintf(buffer, 1024, "-display");
 	instance->td->argv[i++] = strdup(buffer);
@@ -574,6 +620,7 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 
 
     pthread_mutex_lock(&(instance->control_mutex));
+    instance->js_state = JS_STATE_UNDEFINED;
     launchPlayerThread(instance);
     instance->threadsetup = 1;
     pthread_mutex_unlock(&(instance->control_mutex));
@@ -644,11 +691,12 @@ void refresh_frame(char *buffer, _ThreadData * td, Node * node)
 
 //return true if we should try to play this again immediately, false if not 
 
-int playNode(ThreadData * local_td, Node * local_list, char *local_url,
-	     int local_mmsstream, int *usefps, int *nomouseinput,
-	     int *maybeplaylist)
+PlayResult *playNode(ThreadData * local_td, Node * local_list,
+		     char *local_url, int local_mmsstream, int *usefps,
+		     int *nomouseinput, int *maybeplaylist)
 {
 
+    PlayResult *result = (PlayResult *) NPN_MemAlloc(sizeof(PlayResult));
     char buffer[1024];
     char message[1024];
     int notfound;
@@ -656,7 +704,6 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
     char vm[10];
     char *cf;
     double cfpercent;
-    int retval = FALSE;
     char *eos;
     char *msg;
 
@@ -665,15 +712,17 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
     int zerocfbytes_count = 0;
     char url_copy[1024];
     int c;
+    int length_request_count = 0;
+    float lastmedialength = -1.0;
+    int lastmediapercent = -1;
+    int lastpercent = -1;
 #ifdef GTK_ENABLED
     int fsupdated = 0;
 #endif
-#define TRYAGAIN_FALSE 0
-#define TRYAGAIN_TRUE 1
-#define TRYAGAIN_FALLBACK 2
-
-    int tryagain = TRYAGAIN_FALLBACK;
 
+    result->errorcode = ERROR_NO_ERROR;
+    result->tryagain = TRYAGAIN_FALLBACK;
+    result->retval = FALSE;
 
 /*
   the meaning of the above is as follows:
@@ -692,9 +741,18 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 			 (void *) &(local_td->instance->control_mutex));
     pthread_mutex_lock(&(local_td->instance->control_mutex));
     sendCommand(local_td->instance, "get_time_length");
+    local_td->instance->mediaLength = 0.0;
     pthread_mutex_unlock(&(local_td->instance->control_mutex));
     pthread_cleanup_pop(0);
+#ifdef GTK_ENABLED
+	g_idle_add(gtkgui_save_enable, local_td->instance);
+//	g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+#endif
 
+    // set this to 0 for every new media file;
+    local_td->instance->movie_height = 0;
+    local_td->instance->movie_width = 0;
+    
     while (1) {
 	pthread_testcancel();
 	if (local_td->instance->player == NULL)
@@ -713,7 +771,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 	pthread_testcancel();
 #ifdef GTK_ENABLED
-	g_idle_add(gtkgui_save_enable, local_td->instance);
+//	g_idle_add(gtkgui_save_enable, local_td->instance);
+//	g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
 #endif
 
 	pthread_testcancel();
@@ -730,8 +789,18 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    i = 0;
 	    do {
 		pthread_testcancel();
-		c = fgetc(local_td->instance->player);
-		if (c == EOF) {
+	    	// need to lock around the read
+		pthread_mutex_lock(&(local_td->instance->read_mutex));
+
+	    	if ((local_td->instance->cancelled == 0) && (local_td->instance->player != NULL)) {
+			// c = fgetc(local_td->instance->player);
+			fread(&c,1,1,local_td->instance->player);
+		} else {
+			c = EOF;
+		}
+		pthread_mutex_unlock(&(local_td->instance->read_mutex));
+
+	    	if (c == EOF) {
 		    buffer[i] = '\0';
 		    break;
 		}
@@ -780,6 +849,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    // Only update the display if there is something worth displaying
 	    if (strstr(eos, "Cache fill:") != NULL) {
 		cf = strstr(eos, "Cache fill:");
+	    	lastpercent = (int)cfpercent;
 		i = sscanf(cf, "Cache fill: %lf %% (%ld bytes)",
 			   &cfpercent, &cfbytes);
 		//the following is a workaround for an mplayer bug
@@ -794,13 +864,13 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		}
 		if (zerocfbytes_count >= 1 &&
 		    ((strncmp(local_url, "mms://", 6) == 0) ||
-		     local_mmsstream
-		     && strncmp(local_url, "http://", 7) == 0)) {
+		     (local_mmsstream
+		      && strncmp(local_url, "http://", 7) == 0))) {
 
 		    if (DEBUG) {
 			printf("Exiting. Will try again with mmst://\n");
 		    }
-		    tryagain = TRYAGAIN_FALLBACK;
+		    result->tryagain = TRYAGAIN_FALLBACK;
 		    break;
 		}
 
@@ -814,10 +884,17 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 #ifdef GTK_ENABLED
 		snprintf(local_td->instance->lastmessage, 1024, "%s",
 			 message);
-		g_idle_add(gtkgui_message, local_td->instance);
-		local_td->instance->percent = (cfpercent / 100.0);
-		g_idle_add(gtkgui_progress, local_td->instance);
+
+	    	local_td->instance->percent = (cfpercent / 100.0);
+		if (lastpercent != cfpercent) {
+			g_idle_add(gtkgui_message, local_td->instance);
+			g_idle_add(gtkgui_progress, local_td->instance);
+		}
 #endif
+	    pthread_mutex_lock(&(local_td->instance->control_mutex));
+	    local_td->instance->js_state = JS_STATE_BUFFERING;
+	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
+	    
 	    }
 	}
 
@@ -833,6 +910,9 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
 	    g_idle_add(gtkgui_message, local_td->instance);
 #endif
+	    pthread_mutex_lock(&(local_td->instance->control_mutex));
+	    local_td->instance->js_state = JS_STATE_PLAYING;
+	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
 	}
 
 	pthread_testcancel();
@@ -852,7 +932,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 			local_td->instance->embed_height -
 			local_list->play_y;
 
-		if (local_td->instance->player_window != 0) {
+		if (local_td->instance->player_window != 0 && local_td->instance->movie_height != local_list->play_y) { 
 
 		    local_td->instance->movie_height = local_list->play_y;
 		    local_td->instance->movie_width = local_list->play_x;
@@ -869,9 +949,9 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 	    } else {
 		local_td->instance->panel_height = 16;
-		local_td->instance->movie_height = local_list->play_y;
-		local_td->instance->movie_width = local_list->play_x;
-		if (local_td->instance->player_window != 0) {
+		if (local_td->instance->player_window != 0 && local_td->instance->movie_height != local_list->play_y) {
+		    local_td->instance->movie_height = local_list->play_y;
+		    local_td->instance->movie_width = local_list->play_x;
 
 #ifdef GTK_ENABLED
 		    g_idle_add(gtkgui_resize, local_td->instance);
@@ -906,6 +986,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	pthread_testcancel();
 	// mplayer answer back messages
 	if (strstr(buffer, "ANS_LENGTH") != 0) {
+	    if ((int) local_td->instance->mediaLength != 0)
+		lastmedialength = local_td->instance->mediaLength;
 	    msg = strstr(buffer, "ANS_LENGTH");
 	    sscanf(msg, "ANS_LENGTH=%f",
 		   &(local_td->instance->mediaLength));
@@ -914,23 +996,50 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		       local_td->instance->mediaLength);
 	}
 
+	pthread_testcancel();
+	// mplayer answer back messages
+	if (strstr(buffer, "ANS_TIME_POSITION") != 0) {
+	    msg = strstr(buffer, "ANS_TIME_POSITION");
+	    sscanf(msg, "ANS_TIME_POSITION=%f",
+		   &(local_td->instance->mediaPos));
+	    if (DEBUG > 1)
+		printf("Media Position = %f\n",
+		       local_td->instance->mediaPos);
+	    if ((int) local_td->instance->mediaLength > 1) {
+	    	lastmediapercent = local_td->instance->mediaPercent;
+		local_td->instance->mediaPercent =
+		    (int) ((local_td->instance->mediaPos * 100) /
+			   local_td->instance->mediaLength);
+#ifdef GTK_ENABLED
+	    	if (lastmediapercent != local_td->instance->mediaPercent) {
+		    g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
+		    g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+		}
+#endif
+	    }
+	}
+
 	pthread_testcancel();
 	if (strstr(buffer, "ANS_PERCENT_POSITION") != 0) {
 	    msg = strstr(buffer, "ANS_PERCENT_POSITION");
 	    sscanf(msg, "ANS_PERCENT_POSITION=%i",
 		   &(local_td->instance->mediaPercent));
 	    if (local_td->instance->mediaPercent == 0) {
-		if (local_td->instance->mediaLength != 0)
+		if ((int) local_td->instance->mediaLength > 1)
+		    lastmediapercent = local_td->instance->mediaPercent;
 		    local_td->instance->mediaPercent =
-			(int) ((local_td->instance->mediaTime * 100) /
+			(int) ((local_td->instance->mediaPos * 100) /
 			       local_td->instance->mediaLength);
+#ifdef GTK_ENABLED
+		if (lastmediapercent != local_td->instance->mediaPercent) {
+		    g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
+		    g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+		}
+#endif
 	    }
 	    if (DEBUG > 1)
 		printf("Percent Played = %i\n",
 		       local_td->instance->mediaPercent);
-#ifdef GTK_ENABLED
-	    g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
-#endif
 	}
 
 	pthread_testcancel();
@@ -944,7 +1053,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		    printf("mediaTime = %f\n",
 			   local_td->instance->mediaTime);
 #ifdef GTK_ENABLED
-		g_idle_add(gtkgui_draw, local_td->instance);
+//		g_idle_add(gtkgui_draw, local_td->instance);
 #endif
 	    } else {
 		msg = strstr(buffer, "\rV:");
@@ -968,13 +1077,23 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
 	    pthread_cleanup_pop(0);
+	    pthread_testcancel();
 	    if (local_td->instance->paused == 0) {
 		pthread_cleanup_push((void (*)(void *))
 				     pthread_mutex_unlock,
 				     (void *) &(local_td->instance->
 						control_mutex));
 		pthread_mutex_lock(&(local_td->instance->control_mutex));
-		sendCommand(local_td->instance, "get_percent_pos");
+		if ((int) local_td->instance->mediaLength > 1)
+		    sendCommand(local_td->instance, "get_time_pos");
+		if (local_td->instance->mediaLength > lastmedialength) {
+		    sendCommand(local_td->instance, "get_time_length");
+		} else {
+		    if (length_request_count < 10) {
+			sendCommand(local_td->instance, "get_time_length");
+			length_request_count++;
+		    }
+		}
 		pthread_mutex_unlock(&(local_td->instance->control_mutex));
 		pthread_cleanup_pop(0);
 #ifdef GTK_ENABLED
@@ -985,7 +1104,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		}
 #endif
 	    }
-	    tryagain = TRYAGAIN_FALSE;
+	    result->tryagain = TRYAGAIN_FALSE;
 	}
 
 	pthread_testcancel();
@@ -995,6 +1114,23 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    DrawUI(local_td->instance->widget,
 		   local_td->instance, message, 0, -1);
 #endif
+#ifdef GTK_ENABLED
+	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
+	    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+	    pthread_mutex_lock(&(local_td->instance->control_mutex));
+	    local_td->instance->js_state = JS_STATE_WAITING;
+	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
+	
+	}
+
+	pthread_testcancel();
+	if (strstr(buffer, "Initiated") != NULL) {
+	    snprintf(message, 1024, "%s", buffer);
+#ifdef X_ENABLED
+	    DrawUI(local_td->instance->widget,
+		   local_td->instance, message, 0, -1);
+#endif
 #ifdef GTK_ENABLED
 	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
 	    g_idle_add(gtkgui_message, local_td->instance);
@@ -1012,32 +1148,34 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
 	    g_idle_add(gtkgui_message, local_td->instance);
 #endif
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_NO_STREAM;
+	    break;
 	}
 
 
 	pthread_testcancel();
 	if (strstr(buffer, "Example: mplayer -playlist ") != NULL) {
 	    if (*maybeplaylist == 0) {
-		tryagain = TRYAGAIN_TRUE;
+		result->tryagain = TRYAGAIN_TRUE;
 		*maybeplaylist = 1;
-	    } else
-		tryagain = TRYAGAIN_FALSE;
+	    } else {
+		result->tryagain = TRYAGAIN_FALSE;
+	    	result->errorcode = ERROR_NOT_PLAYLIST;
+	    }
 	    break;
 	}
 
 	pthread_testcancel();
-	if (strstr(buffer, "Stream not seekable") != NULL) {
-	    if (*maybeplaylist == 0) {
-		tryagain = TRYAGAIN_TRUE;
-		*maybeplaylist = 1;
-	    } else
-		tryagain = TRYAGAIN_FALSE;
+	if (strstr(buffer, "File not found") != NULL) {
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_FILE_NOT_FOUND;
 	    break;
 	}
 
 	pthread_testcancel();
 	if (strstr(buffer, "Error while decoding") != NULL) {
-	    if (!isMms(local_url)) {
+	    if (!isMms(local_url, local_td->instance->nomediacache)) {
 		if (DEBUG)
 		    printf("Resetting stream, 1 sec rewind\n");
 		pthread_suspend(1000);
@@ -1058,10 +1196,30 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    if (DEBUG)
 		printf("----player thread: breaking read loop - Quit\n");
 	    local_td->instance->js_state = JS_STATE_UNDEFINED;
-	    tryagain = TRYAGAIN_FALSE;
+	    result->tryagain = TRYAGAIN_FALSE;
 	    break;
 	}
 
+	pthread_testcancel();
+	if (strstr
+	    (buffer,
+	     "MPlayer interrupted by signal 13 in module: demux_open") !=
+	    NULL) {
+	    if (DEBUG)
+		printf
+		    ("----player thread: breaking read loop - demux_open\n");
+	    local_td->instance->js_state = JS_STATE_UNDEFINED;
+	    if (local_td->instance->rtsp_use_tcp == 1) {
+		local_td->instance->rtsp_use_tcp = 0;
+		result->tryagain = TRYAGAIN_TRUE;
+	    } else {
+		result->tryagain = TRYAGAIN_FALSE;
+		result->errorcode = ERROR_PLAYER_INTERRUPTED;
+	    }
+	    break;
+	}
+
+
 	pthread_testcancel();
 	// detect Quicktime file with old codec
 	if (strstr(buffer, "MOV: missing header (moov/cmov) chunk!") !=
@@ -1070,15 +1228,21 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		printf
 		    ("----player thread: waiting to download entire movie\n");
 	    }
-	    tryagain = TRYAGAIN_TRUE;
+	    result->tryagain = TRYAGAIN_TRUE;
 	    while (1) {
 		pthread_testcancel();
 		usleep(100);
 		pthread_testcancel();
+		pthread_mutex_lock(&(local_td->instance->control_mutex));
+		local_td->instance->js_state = JS_STATE_BUFFERING;
+		pthread_mutex_unlock(&(local_td->instance->control_mutex));
 		pthread_mutex_lock(&(local_td->instance->playlist_mutex));
 		local_td->instance->state = STATE_DOWNLOADING;
 		if (local_list->retrieved == 1) {
 		    local_td->instance->state = STATE_PLAYING;
+		    pthread_mutex_lock(&(local_td->instance->control_mutex));
+		    local_td->instance->js_state = JS_STATE_PLAYING;
+		    pthread_mutex_unlock(&(local_td->instance->control_mutex));
 		    pthread_mutex_unlock(&
 					 (local_td->instance->
 					  playlist_mutex));
@@ -1093,7 +1257,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 	pthread_testcancel();
 	if (strstr(buffer, "FPS not specified") != NULL) {
-	    tryagain = TRYAGAIN_TRUE;
+	    result->tryagain = TRYAGAIN_TRUE;
 	    *usefps = 1;
 	    break;
 	}
@@ -1101,16 +1265,9 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	pthread_testcancel();
 	if (strstr(buffer, "nomouseinput") != NULL) {
 	    *nomouseinput = 0;
-	    tryagain = TRYAGAIN_TRUE;
+	    result->tryagain = TRYAGAIN_TRUE;
 	    break;
 	}
-// Possible work around for cinema now site     
-//      if (strstr(buffer, "ASF file format detected") != NULL) {
-//          tryagain = TRYAGAIN_TRUE;
-//          local_list->playlist = 1;
-//          break;
-//      }
-
 
 	pthread_testcancel();
 	if (strstr(buffer, "Exiting") != NULL) {
@@ -1123,7 +1280,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
 #endif
 	    }
-	    tryagain = TRYAGAIN_FALSE;
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_NO_ERROR;
 	    break;
 	}
 
@@ -1133,6 +1291,37 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    if (DEBUG)
 		printf
 		    ("----player thread: breaking read loop - interrupted\n");
+	    if (strstr(buffer, "video_read_frame") != NULL) {
+		local_td->instance->js_state = JS_STATE_UNDEFINED;
+		if (local_td->instance->rtsp_use_tcp == 1) {
+		    local_td->instance->rtsp_use_tcp = 0;
+		    result->tryagain = TRYAGAIN_TRUE;
+		} else {
+#ifdef GTK_ENABLED
+		    snprintf(local_td->instance->lastmessage, 1024,
+			     "Error: %s", buffer);
+		    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+		    result->tryagain = TRYAGAIN_FALSE;
+		    result->errorcode = ERROR_PLAYER_INTERRUPTED;
+		}
+	    }
+	    break;
+	}
+
+	pthread_testcancel();
+        if (strstr(buffer, "execv") != NULL) {
+	    if (DEBUG)
+		printf
+		    ("----player thread: breaking read loop - execv failed\n");
+	    local_td->instance->js_state = JS_STATE_UNDEFINED;
+#ifdef GTK_ENABLED
+	    snprintf(local_td->instance->lastmessage, 1024, "Error: %s %s",
+		     local_td->argv[0], buffer);
+	    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_EXECV;
 	    break;
 	}
 
@@ -1140,63 +1329,75 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	if (strstr(buffer, "explicit kill") != NULL) {
 	    if (DEBUG)
 		printf("----player thread: breaking read loop - killed\n");
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_EXPLICIT_KILL;
 	    break;
 	}
 
 	pthread_testcancel();
+/*	commmented out to make BBC site work    
 	if (strstr(buffer, "everything done") != NULL) {
 	    if (DEBUG)
 		printf
 		    ("----player thread: breaking read loop - codec issue\n");
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_CODEC_FAILURE;
 	    break;
 	}
-
+*/
 
 	assert(local_td->instance->player != NULL);
 
     }				// end of READ LOOP
 
     if (DEBUG) {
-	printf("----player thread: tryagain = %d\n", tryagain);
+	printf("----player thread: tryagain = %d\n", result->tryagain);
     }
     //we always return either true or false
-    if (tryagain == TRYAGAIN_TRUE) {
-	retval = TRUE;
+    if (result->tryagain == TRYAGAIN_TRUE) {
+	result->retval = TRUE;
     }
 
-    if (tryagain == TRYAGAIN_FALSE) {
-	retval = FALSE;
+    if (result->tryagain == TRYAGAIN_FALSE) {
+	result->retval = FALSE;
     }
 
-    if (tryagain == TRYAGAIN_FALLBACK) {
+    if (result->tryagain == TRYAGAIN_FALLBACK) {
 	//fallback to msst if we can
 	strlcpy(url_copy, local_url, 1023);
 	url_copy[1023] = '\0';
 
 	if (strncmp(local_url, "mms://", 6) == 0) {
 	    snprintf(local_url, 1023, "mmst://%s", url_copy + 6);
-	    retval = TRUE;
+	    result->retval = TRUE;
 	} else if (local_mmsstream &&
 		   strncmp(local_url, "http://", 7) == 0) {
 
 	    snprintf(local_url, 1023, "mmst://%s", url_copy + 7);
-	    retval = TRUE;
+	    result->retval = TRUE;
 	} else {
-	    retval = FALSE;
+	    result->retval = FALSE;
 	}
     }
-
-    return retval;
+#ifdef GTK_ENABLED
+    if (strstr(local_td->instance->lastmessage, "Error") == NULL) {
+	snprintf(local_td->instance->lastmessage, 1024, _("Stopped"));
+	g_idle_add(gtkgui_message, local_td->instance);
+    }
+#endif
+    return result;
 }
 
 void *playPlaylist(void *td)
 {
 
+    PlayResult *result = NULL;
     FILE *playlist;
     char cmd[2048];
     char message[1024];
     char mmsplaylist[1024];
     char buffer[1024];		// scratch pad
+    char *p;			// pointer to find the ( in the callback
 
     int argc = 0, base_argc = 0;
     ThreadData *local_td;
@@ -1213,6 +1414,7 @@ void *playPlaylist(void *td)
     int nomouseinput;
     int maybeplaylist;
     int listempty;
+    int lasterror = -1;
 
     local_td = (ThreadData *) td;
     local_list = local_td->list;
@@ -1221,7 +1423,7 @@ void *playPlaylist(void *td)
 	pthread_exit(0);
 
     if (DEBUG)
-	printf("----player thread: in playPlayList\n");
+	printf("----player thread: in playPlaylist\n");
 
     pthread_cleanup_push((void (*)(void *)) pthread_mutex_unlock,
 			 (void *) &(local_td->instance->
@@ -1241,13 +1443,14 @@ void *playPlaylist(void *td)
     if (local_td->instance->state < STATE_STARTED_PLAYER) {
 	// wait for playlist_complete_cond to be signalled
 	// this should happen when we have completed getting all the playlist elements
+	local_td->instance->state = STATE_WAITING_FOR_SIGNAL;
 	pthread_cond_wait(&(local_td->instance->playlist_complete_cond),
 			  &(local_td->instance->playlist_cond_mutex));
 	pthread_mutex_unlock(&(local_td->instance->playlist_cond_mutex));
 
 	pthread_testcancel();
 	sleep(1);
-	// playlist_cond_mutex should be locked now
+	// playlist_cond_mutex should be unlocked now
 	if (local_td != NULL) {
 	    if (DEBUG > 1)
 		printf("local_td = %p\n", local_td);
@@ -1275,6 +1478,8 @@ void *playPlaylist(void *td)
 		    ("We were signalled to start but local_td is NULL, should not happen\n");
 	    pthread_exit(0);
 	}
+    } else {
+	local_td->instance->threadsignaled = 1;
     }
 
     pthread_cleanup_pop(0);
@@ -1418,6 +1623,34 @@ void *playPlaylist(void *td)
 		printf("----player thread: nothing on the list to play\n");
 	    listempty = 1;
 	    local_mmsstream = 0;
+	    // update MediaCompleteWithError Callback with PLAYLIST EMPTY but only if we have not played anything
+	    if ((local_td->instance->mediaCompleteWithErrorCallback !=
+		 NULL) && (lasterror == -1)) {
+		if (DEBUG)
+		    printf("mediaCompleteWithError(before) = %s\n",
+			   local_td->instance->
+			   mediaCompleteWithErrorCallback);
+		strlcpy(message,
+			local_td->instance->mediaCompleteWithErrorCallback,
+			1024);
+		p = index(message, '(');
+		if (p == NULL) {
+		    p = message + strlen(message);
+		}
+		*p = '\0';
+		snprintf(buffer, 1024, "%s(%i);", message,
+			 ERROR_PLAYLIST_EMPTY);
+		NPN_MemFree(local_td->instance->
+			    mediaCompleteWithErrorCallback);
+		local_td->instance->mediaCompleteWithErrorCallback =
+		    (char *) NPN_MemAlloc(strlen(buffer));
+		strlcpy(local_td->instance->mediaCompleteWithErrorCallback,
+			buffer, strlen(buffer));
+		if (DEBUG)
+		    printf("mediaCompleteWithError(after) = %s\n",
+			   local_td->instance->
+			   mediaCompleteWithErrorCallback);
+	    }
 	} else {
 	    //we have a node to play
 	    if (DEBUG)
@@ -1442,6 +1675,11 @@ void *playPlaylist(void *td)
 	maybeplaylist = 0;
 
 	do {
+	    if (result != NULL) {
+		lasterror = result->errorcode;
+		NPN_MemFree(result);
+		result = NULL;
+	    }
 
 	    pthread_testcancel();
 
@@ -1482,6 +1720,11 @@ void *playPlaylist(void *td)
 		usefps = 0;
 	    }
 
+	    if (local_td->instance->rtsp_use_tcp == 1) {
+		snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
+		local_td->argv[argc++] = strdup(buffer);
+	    }
+
 	    if (nomouseinput) {
 		local_td->argv[argc++] = strdup("-nomouseinput");
 //          } else {
@@ -1521,15 +1764,17 @@ void *playPlaylist(void *td)
 		}
 	    } else {
 		if (strlen(local_list->fname) == 0) {
+		    if (local_list->playlist == 1 || maybeplaylist != 0)
+			local_td->argv[argc++] = strdup("-playlist");
 		    local_td->argv[argc++] = strdup(local_url);
 		} else {
 		    local_td->argv[argc++] = strdup("-nocache");
+		    if (local_list->playlist == 1 || maybeplaylist != 0)
+			local_td->argv[argc++] = strdup("-playlist");
 		    local_td->argv[argc++] = strdup(local_list->fname);
 		}
 	    }
 
-	    if (local_list->playlist == 1 || maybeplaylist != 0)
-		local_td->argv[argc++] = strdup("-playlist");
 
 	    if (DEBUG) {
 		printf("----player thread: URL: %s\n", local_url);
@@ -1549,7 +1794,8 @@ void *playPlaylist(void *td)
 
 	    local_td->instance->player =
 		mypopen(local_td->argv, &(local_td->instance->pid),
-			&(local_td->instance->control));
+			&(local_td->instance->control),
+			local_td->instance);
 	    if (local_td->instance->player != NULL) {
 		local_td->instance->js_state = JS_STATE_PLAYING;
 	    }
@@ -1566,15 +1812,19 @@ void *playPlaylist(void *td)
 		local_td->instance->mediaLength = 0.0;
 		local_td->instance->mediaPercent = 0;
 
-		tryagain =
+		result =
 		    playNode(local_td, local_list, local_url,
 			     local_mmsstream, &usefps, &nomouseinput,
 			     &maybeplaylist);
 
-		if (DEBUG)
+		pthread_mutex_lock(&(local_td->instance->control_mutex));
+		local_td->instance->js_state = JS_STATE_TRANSITIONING;
+		pthread_mutex_unlock(&(local_td->instance->control_mutex));
+
+ 	        if (DEBUG)
 		    printf
 			("----player thread: playNode returned = %d\n",
-			 tryagain);
+			 result->retval);
 
 		pthread_testcancel();
 		assert(local_list != NULL);
@@ -1586,9 +1836,9 @@ void *playPlaylist(void *td)
 
 		// this cancels the download of this media
 		// if there are more than 1, saves bandwidth
-		//also revents us from playing it again
+		// also revents us from playing it again
 		pthread_mutex_lock(&(local_td->instance->playlist_mutex));
-		if (!tryagain) {
+		if (!(result->retval)) {
 		    local_list->played = 1;
 		}
 		pthread_mutex_unlock(&
@@ -1597,9 +1847,6 @@ void *playPlaylist(void *td)
 		assert(local_td->instance != NULL);
 
 		//close the pipes to mplayer
-		pthread_mutex_lock(&(local_td->instance->control_mutex));
-		local_td->instance->js_state = JS_STATE_TRANSITIONING;
-		pthread_mutex_unlock(&(local_td->instance->control_mutex));
 
 
 		assert(local_td->instance->control > 0);
@@ -1616,7 +1863,35 @@ void *playPlaylist(void *td)
 		local_td->instance->state = STATE_PLAYLIST_NEXT;
 
 	    }
-	} while (tryagain);
+	    // update MediaCompleteWithError Callback
+	    if (local_td->instance->mediaCompleteWithErrorCallback != NULL) {
+		if (DEBUG)
+		    printf("mediaCompleteWithError(before) = %s\n",
+			   local_td->instance->
+			   mediaCompleteWithErrorCallback);
+		strlcpy(message,
+			local_td->instance->mediaCompleteWithErrorCallback,
+			1024);
+		p = index(message, '(');
+		if (p == NULL) {
+		    p = message + strlen(message);
+		}
+		*p = '\0';
+		snprintf(buffer, 1024, "%s(%i);", message,
+			 result->errorcode);
+		NPN_MemFree(local_td->instance->
+			    mediaCompleteWithErrorCallback);
+		local_td->instance->mediaCompleteWithErrorCallback =
+		    (char *) NPN_MemAlloc(strlen(buffer));
+		strlcpy(local_td->instance->mediaCompleteWithErrorCallback,
+			buffer, strlen(buffer));
+		if (DEBUG)
+		    printf("mediaCompleteWithError(after) = %s\n",
+			   local_td->instance->
+			   mediaCompleteWithErrorCallback);
+	    }
+
+	} while (result->retval);
 
 	local_td->instance->currentnode = NULL;
 
@@ -1656,7 +1931,20 @@ void *playPlaylist(void *td)
 	g_idle_add(mediacallback, local_td->instance);
     }
 #endif
-
+#ifdef X_ENABLED
+    if (local_td->instance->mediaCompleteCallback != NULL)
+	NPN_GetURL(local_td->instance->mInstance,
+		   local_td->instance->mediaCompleteCallback, "_self");
+    if (local_td->instance->mediaCompleteWithErrorCallback != NULL)
+	NPN_GetURL(local_td->instance->mInstance,
+		   local_td->instance->mediaCompleteWithErrorCallback,
+		   "_self");
+#endif
+#ifdef GTK2_ENABLED
+    if (GTK_IS_WIDGET(local_td->instance->src_event_box)) {
+	gtk_widget_show(local_td->instance->src_event_box);
+    }
+#endif
     if (DEBUG) {
 	printf("----player thread: callbacks complete\n");
     }
@@ -1683,8 +1971,7 @@ void *playPlaylist(void *td)
     if (DEBUG) {
 	printf("----player thread: normal exit\n");
     }
-
-    pthread_exit(0);
+    //pthread_exit(0);
 
     return NULL;
 }
