@@ -30,6 +30,9 @@ extern int DEBUG;
 #include "../pixmaps/rew_up_small.xpm"
 #include "../pixmaps/fs_down_small.xpm"
 #include "../pixmaps/fs_up_small.xpm"
+
+// default src button
+#include "../pixmaps/start.xpm"
 #endif
 
 #define BUTTON_WIDTH	43
@@ -469,6 +472,7 @@ void play_callback(GtkWidget * widget, GdkEventExpose * event,
 	if (instance->panel_height > 16)
 	    instance->panel_height = 16;
 
+	instance->redrawbuttons = 1;
 	gtk_container_remove(GTK_CONTAINER(instance->play_event_box),
 			     instance->image_play);
 	gtk_container_remove(GTK_CONTAINER(instance->pause_event_box),
@@ -508,9 +512,13 @@ void play_callback(GtkWidget * widget, GdkEventExpose * event,
 	    gtk_widget_show(instance->stop_event_box);
 	}
 	gtk_widget_show(instance->fixed_container);
-	if (instance->drawing_area != NULL)
+	if (instance->drawing_area != NULL) {
+	    gtk_widget_hide(GTK_WIDGET(instance->drawing_area));
 	    gtk_widget_show(GTK_WIDGET(instance->drawing_area));
-
+	}
+    	if (GTK_IS_WIDGET(instance->src_event_box) && instance->targetplayer == 0) {
+	    gtk_widget_hide(instance->src_event_box);
+	}
 	gdk_flush();
     }
     if (widget != NULL)
@@ -638,7 +646,7 @@ void stop_callback(GtkWidget * widget, GdkEventExpose * event,
 	gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
 	if (instance->drawing_area != NULL) {
 	    gtk_widget_hide(GTK_WIDGET(instance->drawing_area));
-	    snprintf(instance->lastmessage, 1024, "Stopped");
+	    snprintf(instance->lastmessage, 1024, _("Stopped"));
 	    g_idle_add(gtkgui_message, instance);
 	    gtk_widget_hide(GTK_WIDGET(instance->mediaprogress_bar));
 	}
@@ -846,9 +854,128 @@ gboolean keyboard_callback(GtkWidget * widget, GdkEventKey * event,
     return FALSE;
 }
 
+gboolean mouse_callback(GtkWidget * widget, GdkEventButton * event,
+			nsPluginInstance * instance)
+{
+    char buffer1[1024];
+    char buffer2[1024];
+    char *p;
+
+    if (DEBUG)
+	printf("In mouse_callback\n");
+    if (event->type == GDK_BUTTON_PRESS) {
+	if (DEBUG)
+	    printf("button press # %i\n", event->button);
+	if (event->button == 1) {
+	    if (instance->mouseClickCallback != NULL)
+		NPN_GetURL(instance->mInstance,
+			   instance->mouseClickCallback, "_self");
+	}
+	if (instance->mouseDownCallback != NULL) {
+	    strlcpy(buffer1, instance->mouseDownCallback, 1024);
+	    p = index(buffer1, '(');
+	    if (p == NULL) {
+		p = buffer1 + strlen(buffer1);
+	    }
+	    *p = '\0';
+	    snprintf(buffer2, 1024, "%s(%i);", buffer1, event->button);
+	    NPN_MemFree(instance->mouseDownCallback);
+	    instance->mouseDownCallback =
+		(char *) NPN_MemAlloc(strlen(buffer2));
+	    strlcpy(instance->mouseDownCallback, buffer2, strlen(buffer2));
+	    NPN_GetURL(instance->mInstance,
+		       instance->mouseDownCallback, "_self");
+	}
+    }
+
+    if (event->type == GDK_BUTTON_RELEASE) {
+	if (DEBUG)
+	    printf("button released # %i\n", event->button);
+	if (instance->mouseUpCallback != NULL) {
+	    strlcpy(buffer1, instance->mouseUpCallback, 1024);
+	    p = index(buffer1, '(');
+	    if (p == NULL) {
+		p = buffer1 + strlen(buffer1);
+	    }
+	    *p = '\0';
+	    snprintf(buffer2, 1024, "%s(%i);", buffer1, event->button);
+	    NPN_MemFree(instance->mouseUpCallback);
+	    instance->mouseUpCallback =
+		(char *) NPN_MemAlloc(strlen(buffer2));
+	    strlcpy(instance->mouseUpCallback, buffer2, strlen(buffer2));
+
+	    NPN_GetURL(instance->mInstance,
+		       instance->mouseUpCallback, "_self");
+	}
+    }
+
+    return FALSE;
+}
+
+gboolean mediaprogress_callback(GtkWidget * widget, GdkEventButton * event,
+				nsPluginInstance * instance)
+{
+    GtkRequisition *req;
+    float percent;
+    float bytepercent;
+    float seektime;
+
+    req = (GtkRequisition *) NPN_MemAlloc(sizeof(GtkRequisition));
+    gtk_widget_size_request(widget, req);
+
+
+    percent = event->x / req->width;
+
+    if (instance->currentnode != NULL) {
+	if (instance->currentnode->totalbytes != 0) {
+	    bytepercent =
+		(float) instance->currentnode->bytes /
+		(float) instance->currentnode->totalbytes;
+	    if (percent > bytepercent) {
+		percent = bytepercent - 0.05;
+	    }
+	}
+    }
+    seektime = instance->mediaLength * percent;
+
+
+    if (DEBUG) {
+	printf("widget size: %i x %i\n", req->width, req->height);
+	printf("mouse click at %f x %f\n", event->x, event->y);
+	printf("percent = %f\nseektime = %f\n", percent, seektime);
+    }
+    if (seektime > 0)
+	instance->Seek(seektime);
+    NPN_MemFree(req);
+
+    return TRUE;
+}
+
+
+gboolean mousenotify_callback(GtkWidget * widget, GdkEventCrossing * event,
+			      nsPluginInstance * instance)
+{
+    if (event->type == GDK_ENTER_NOTIFY) {
+	if (instance->mouseEnterCallback != NULL)
+	    NPN_GetURL(instance->mInstance,
+		       instance->mouseEnterCallback, "_self");
+
+    }
+
+    if (event->type == GDK_LEAVE_NOTIFY) {
+	if (instance->mouseLeaveCallback != NULL)
+	    NPN_GetURL(instance->mInstance,
+		       instance->mouseLeaveCallback, "_self");
+
+    }
+
+    return FALSE;
+}
+
 gboolean gtkgui_draw(void *data)
 {
     nsPluginInstance *instance;
+    GtkTooltips *tooltip;
 
     instance = (nsPluginInstance *) data;
 
@@ -873,6 +1000,13 @@ gboolean gtkgui_draw(void *data)
 	win_width = instance->window_width;
     }
 
+    if (instance->targetplayer == 1) {
+	win_height = instance->movie_height + 16;
+	win_width = instance->movie_width;
+	if (instance->showcontrols && instance->movie_height)
+	    instance->panel_height = 16;
+    }
+
     if (instance->panel_drawn == 0) {
 	if (instance->mode == NP_FULL)
 	    instance->panel_height = instance->window_height;
@@ -880,7 +1014,7 @@ gboolean gtkgui_draw(void *data)
 	    instance->panel_height = 16;
 	if (instance->panel_height < 0)
 	    instance->panel_height = 0;
-	if (DEBUG > 1) {
+	if (DEBUG) {
 	    printf("drawing panel\n height = %i panel= %i\n",
 		   win_height, instance->panel_height);
 	    printf("showbuttons = %i\nshowcontrols = %i\n",
@@ -950,6 +1084,9 @@ gboolean gtkgui_draw(void *data)
 	    // rewind
 	    if (instance->mmsstream == 0 && win_width > 126) {
 		instance->rew_event_box = gtk_event_box_new();
+		tooltip = gtk_tooltips_new();
+		gtk_tooltips_set_tip(tooltip, instance->rew_event_box,
+				     _("Rewind"), NULL);
 		gtk_widget_set_events(instance->rew_event_box,
 				      GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
@@ -975,6 +1112,9 @@ gboolean gtkgui_draw(void *data)
 	    }
 	    // play
 	    instance->play_event_box = gtk_event_box_new();
+	    tooltip = gtk_tooltips_new();
+	    gtk_tooltips_set_tip(tooltip, instance->play_event_box,
+				 _("Play"), NULL);
 	    gtk_widget_set_events(instance->play_event_box,
 				  GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
@@ -998,6 +1138,9 @@ gboolean gtkgui_draw(void *data)
 	    }
 	    // pause
 	    instance->pause_event_box = gtk_event_box_new();
+	    tooltip = gtk_tooltips_new();
+	    gtk_tooltips_set_tip(tooltip, instance->pause_event_box,
+				 _("Pause"), NULL);
 	    gtk_widget_set_events(instance->pause_event_box,
 				  GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
@@ -1024,6 +1167,9 @@ gboolean gtkgui_draw(void *data)
 	    }
 	    // stop
 	    instance->stop_event_box = gtk_event_box_new();
+	    tooltip = gtk_tooltips_new();
+	    gtk_tooltips_set_tip(tooltip, instance->stop_event_box,
+				 _("Stop"), NULL);
 	    gtk_widget_set_events(instance->stop_event_box,
 				  GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
@@ -1048,6 +1194,9 @@ gboolean gtkgui_draw(void *data)
 	    // fastforward
 	    if (instance->mmsstream == 0 && win_width > 126) {
 		instance->ff_event_box = gtk_event_box_new();
+		tooltip = gtk_tooltips_new();
+		gtk_tooltips_set_tip(tooltip, instance->ff_event_box,
+				     _("Fast Forward"), NULL);
 		gtk_widget_set_events(instance->ff_event_box,
 				      GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
@@ -1079,11 +1228,26 @@ gboolean gtkgui_draw(void *data)
 				     (instance->mediaprogress_bar),
 				     (win_width -
 				      width * multiplier - 20 - width),
-				     height - 4);
+				     height - 2);
+		gtk_widget_set_events(GTK_WIDGET
+				      (instance->mediaprogress_bar),
+				      GDK_BUTTON_PRESS_MASK);
+#ifdef GTK2_ENABLED
+		g_signal_connect(G_OBJECT(instance->mediaprogress_bar),
+				 "button_press_event",
+				 G_CALLBACK(mediaprogress_callback),
+				 instance);
+#endif
+#ifdef GTK1_ENABLED
+		gtk_signal_connect(GTK_OBJECT(instance->mediaprogress_bar),
+				   "button_press_event",
+				   GTK_SIGNAL_FUNC(mediaprogress_callback),
+				   instance);
+#endif
 		gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			      GTK_WIDGET(instance->mediaprogress_bar),
 			      (width * multiplier + 10),
-			      win_height - height + 2);
+			      win_height - height + 1);
 	    }
 
 	    if (instance->mode == NP_EMBED && instance->noembed == 0)
@@ -1091,6 +1255,9 @@ gboolean gtkgui_draw(void *data)
 
 	    // fullscreen
 	    instance->fs_event_box = gtk_event_box_new();
+	    tooltip = gtk_tooltips_new();
+	    gtk_tooltips_set_tip(tooltip, instance->fs_event_box,
+				 _("Full Screen"), NULL);
 	    gtk_widget_set_events(instance->fs_event_box,
 				  GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
@@ -1115,7 +1282,8 @@ gboolean gtkgui_draw(void *data)
 		}
 	    }
 	    gtk_widget_show(instance->fixed_container);
-	    gtk_widget_show(instance->gtkwidget);
+	    if (instance->targetplayer == 0)
+		gtk_widget_show(instance->gtkwidget);
 
 	    instance->controlsvisible = 1;
 	    instance->panel_drawn = 1;
@@ -1159,7 +1327,9 @@ gboolean gtkgui_progress(void *data)
 	    if ((instance->percent > 0.0) && (instance->percent < 1.0)) {
 		if (instance->movie_height == 0
 		    && instance->movie_width == 0)
-		    gtk_widget_show(GTK_WIDGET(instance->progress_bar));
+		    if (!(instance->hidestatus))
+			gtk_widget_show(GTK_WIDGET
+					(instance->progress_bar));
 		gtk_progress_bar_update(instance->progress_bar,
 					instance->percent);
 	    }
@@ -1195,12 +1365,42 @@ gboolean gtkgui_resize(void *data)
 		    || (instance->movie_height + 100 >=
 			instance->window_height))
 		    if (instance->fullscreen == 0)
-			if (GTK_IS_CONTAINER(instance->fixed_container))
+			if (GTK_IS_CONTAINER(instance->fixed_container)) {
 			    gtk_fixed_put(GTK_FIXED
 					  (instance->fixed_container),
 					  instance->drawing_area, 0, 0);
+			    gtk_widget_hide(instance->drawing_area);
+			    gtk_widget_show(instance->drawing_area);
+			}
+
 	}
     }
+
+
+    if (instance->targetplayer == 1) {
+#ifdef GTK2_ENABLED
+	gtk_widget_set_size_request(instance->gtkwidget,
+				    instance->movie_width,
+				    instance->movie_height);
+	gtk_widget_set_size_request(instance->drawing_area,
+				    instance->movie_width,
+				    instance->movie_height);
+	gtk_window_resize(GTK_WINDOW(instance->gtkwidget),
+			  instance->movie_width,
+			  instance->movie_height + 16);
+#endif
+#ifdef GTK1_ENABLED
+	gtk_widget_set_usize(instance->gtkwidget,
+			     instance->movie_width,
+			     instance->movie_height + 16);
+	gtk_widget_set_usize(instance->drawing_area,
+			     instance->movie_width,
+			     instance->movie_height + 16);
+#endif
+	instance->redrawbuttons = 1;
+	gtkgui_updatebuttons(data);
+    }
+
     return FALSE;
 
 }
@@ -1249,7 +1449,7 @@ gboolean gtkgui_save_enable(void *data)
     if (instance->js_state != JS_STATE_PLAYING)
 	return FALSE;
 
-    if (!isMms(instance->currentnode->url)) {
+    if (!isMms(instance->currentnode->url, instance->nomediacache)) {
 	pthread_mutex_lock(&(instance->playlist_mutex));
 	if (DEBUG > 1)
 	    printf("Save Enable called retrieved = %i\n",
@@ -1293,6 +1493,9 @@ gboolean gtkgui_save_enable(void *data)
 gboolean gtkgui_drawMediaProgress(void *data)
 {
     nsPluginInstance *instance;
+    int hour = 0, min = 0, length_hour = 0, length_min = 0;
+    long int seconds, length_seconds;
+    char display[30];
 
     if (DEBUG > 1)
 	printf("in drawMediaProgress method\n");
@@ -1304,13 +1507,56 @@ gboolean gtkgui_drawMediaProgress(void *data)
 	return FALSE;
     if (instance->mediaprogress_bar != NULL) {
 	if (GTK_IS_PROGRESS_BAR(instance->mediaprogress_bar)) {
-	    if ((instance->mediaPercent > 0)
+	    if (((int) instance->mediaLength > 0)
 		&& (instance->mediaPercent <= 100)) {
 		if (instance->controlsvisible && instance->showtracker)
 		    gtk_widget_show(GTK_WIDGET
 				    (instance->mediaprogress_bar));
 		gtk_progress_bar_update(instance->mediaprogress_bar,
 					instance->mediaPercent / 100.0);
+
+#ifdef GTK2_ENABLED
+		if (((int) instance->mediaTime > 0)
+		    && (instance->showtime == 1)) {
+		    seconds = (int) instance->mediaTime;
+		    if (seconds >= 3600) {
+			hour = seconds / 3600;
+			seconds = seconds - (hour * 3600);
+		    }
+		    if (seconds >= 60) {
+			min = seconds / 60;
+			seconds = seconds - (min * 60);
+		    }
+		    length_seconds = (int) instance->mediaLength;
+		    if (length_seconds >= 3600) {
+			length_hour = length_seconds / 3600;
+			length_seconds =
+			    length_seconds - (length_hour * 3600);
+		    }
+		    if (length_seconds >= 60) {
+			length_min = length_seconds / 60;
+			length_seconds =
+			    length_seconds - (length_min * 60);
+		    }
+		    if (hour == 0 && length_hour == 0) {
+			snprintf(display, 30, "%2i:%02i / %2i:%02i", min,
+				 (int) seconds, length_min,
+				 (int) length_seconds);
+		    } else {
+			snprintf(display, 30,
+				 "%i:%02i:%02i / %i:%02i:%02i", hour, min,
+				 (int) seconds, length_hour, length_min,
+				 (int) length_seconds);
+		    }
+		    gtk_progress_bar_set_text(instance->mediaprogress_bar,
+					      display);
+		} else {
+		    snprintf(display, 30, "");
+		    gtk_progress_bar_set_text(instance->mediaprogress_bar,
+					      display);
+		}
+#endif
+	    
 	    } else {
 		gtk_widget_hide(GTK_WIDGET(instance->mediaprogress_bar));
 	    }
@@ -1325,18 +1571,64 @@ gboolean gtkgui_drawMediaProgress(void *data)
     return FALSE;
 }
 
-gboolean gtkgui_refreshbuttonstate(void *data) 
+gboolean gtkgui_refreshbuttonstate(void *data)
 {
     nsPluginInstance *instance;
 
     if (DEBUG > 1)
 	printf("in refreshbuttonstate method\n");
-
     instance = (nsPluginInstance *) data;
     if (instance == NULL)
 	return FALSE;
     if (instance->mInitialized == FALSE)
 	return FALSE;
+    if (instance->panel_drawn == 1 && instance->controlsvisible) {
+
+	if (instance->mmsstream == 0) {
+	    if (instance->rew_event_box != NULL) {
+		if (instance->showcontrols && instance->showbuttons
+		    && instance->controlsvisible) {
+		    if (instance->redrawbuttons)
+			gtk_widget_hide(GTK_WIDGET
+					(instance->rew_event_box));
+		    gtk_widget_show(GTK_WIDGET(instance->rew_event_box));
+		}
+	    }
+	} else {
+	    if (GTK_IS_WIDGET(instance->rew_event_box))
+		gtk_widget_hide(GTK_WIDGET(instance->rew_event_box));
+	}
+	if (instance->mmsstream == 0) {
+	    if (instance->ff_event_box != NULL) {
+		if (instance->showcontrols && instance->showbuttons
+		    && instance->controlsvisible) {
+		    if (instance->redrawbuttons)
+			gtk_widget_hide(GTK_WIDGET
+					(instance->ff_event_box));
+		    gtk_widget_show(GTK_WIDGET(instance->ff_event_box));
+		}
+	    }
+	} else {
+	    if (GTK_IS_WIDGET(instance->ff_event_box))
+		gtk_widget_hide(GTK_WIDGET(instance->ff_event_box));
+	}
+	if (instance->showcontrols && instance->showbuttons
+	    && instance->controlsvisible) {
+	    if (instance->redrawbuttons) {
+		gtk_widget_hide(GTK_WIDGET(instance->play_event_box));
+		gtk_widget_hide(GTK_WIDGET(instance->pause_event_box));
+		gtk_widget_hide(GTK_WIDGET(instance->stop_event_box));
+		gtk_widget_hide(GTK_WIDGET(instance->fs_event_box));
+	    }
+	    gtk_widget_show(GTK_WIDGET(instance->play_event_box));
+	    gtk_widget_show(GTK_WIDGET(instance->pause_event_box));
+	    gtk_widget_show(GTK_WIDGET(instance->stop_event_box));
+	    gtk_widget_show(GTK_WIDGET(instance->fs_event_box));
+	}
+    }
+
+    instance->redrawbuttons = 0;
+//    gtk_widget_queue_draw_area(instance->gtkwidget,0,0,instance->embed_width,instance->embed_height);
 
     return FALSE;
 }
@@ -1365,6 +1657,13 @@ gboolean gtkgui_updatebuttons(void *data)
 	win_width = instance->window_width;
     }
 
+    if (instance->targetplayer) {
+	win_height = instance->movie_height + 16;
+	win_width = instance->movie_width;
+    }
+
+    printf("buttons are at %i x %i\n", win_height, win_width);
+
     if (instance->panel_drawn == 1) {
 	height = 16;
 	width = 21;
@@ -1378,7 +1677,7 @@ gboolean gtkgui_updatebuttons(void *data)
 		    gtk_fixed_move(GTK_FIXED(instance->fixed_container),
 				   GTK_WIDGET(instance->rew_event_box),
 				   width * multiplier++,
-				   instance->embed_height - height);
+				   win_height - height);
 		}
 	    }
 	} else {
@@ -1628,10 +1927,11 @@ gint CloseConfig(GtkWidget * widget, nsPluginInstance * instance)
 
 }
 
-gint CloseConfigEvent(GtkWidget * widget, GdkEvent *event, nsPluginInstance * instance)
+gint CloseConfigEvent(GtkWidget * widget, GdkEvent * event,
+		      nsPluginInstance * instance)
 {
 
-    if (GTK_IS_WIDGET(instance->conf_window)) 
+    if (GTK_IS_WIDGET(instance->conf_window))
 	gtk_widget_destroy(instance->conf_window);
     instance->conf_window = NULL;
     return (FALSE);
@@ -1653,6 +1953,7 @@ gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
     instance->cache_percent =
 	(int) gtk_range_get_value(GTK_RANGE(instance->conf_cachepercent));
 
+#ifdef GTK24_ENABLED
     if (instance->vo != NULL)
 	free(instance->vo);
     instance->vo =
@@ -1670,6 +1971,46 @@ gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
     instance->download_dir =
 	strdup(gtk_entry_get_text
 	       (GTK_ENTRY(GTK_BIN(instance->conf_dir)->child)));
+#endif
+    
+    instance->showtime =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_showtime));
+    instance->enable_wmp =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_wmp));
+    instance->enable_qt =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_qt));
+    instance->enable_rm =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_rm));
+    instance->enable_gmp =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_gmp));
+    instance->enable_mpeg =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_mpeg));
+    instance->enable_mp3 =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_mp3));
+    instance->enable_ogg =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_ogg));
+    instance->enable_smil =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_smil));
+    instance->enable_helix =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_enable_helix));
+
+    instance->nomediacache =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_nomediacache));
+
+    instance->rtsp_use_tcp =
+	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+				     (instance->conf_rtsptcp));
 
     gtk_widget_destroy(instance->conf_window);
     instance->conf_window = NULL;
@@ -1702,6 +2043,19 @@ gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
 	fprintf(output, "cache-percent=%i\n", instance->cache_percent);
 	if (strlen(instance->download_dir) > 0)
 	    fprintf(output, "dload-dir=%s\n", instance->download_dir);
+	fprintf(output, "showtime=%i\n", instance->showtime);
+	fprintf(output, "enable-wmp=%i\n", instance->enable_wmp);
+	fprintf(output, "enable-qt=%i\n", instance->enable_qt);
+	fprintf(output, "enable-rm=%i\n", instance->enable_rm);
+	fprintf(output, "enable-gmp=%i\n", instance->enable_gmp);
+	fprintf(output, "enable-mpeg=%i\n", instance->enable_mpeg);
+	fprintf(output, "enable-mp3=%i\n", instance->enable_mp3);
+	fprintf(output, "enable-ogg=%i\n", instance->enable_ogg);
+	fprintf(output, "enable-smil=%i\n", instance->enable_smil);
+	fprintf(output, "enable-helix=%i\n", instance->enable_helix);
+	fprintf(output, "nomediacache=%i\n", instance->nomediacache);
+	fprintf(output, "rtsp-use-tcp=%i\n", instance->rtsp_use_tcp);
+
 
 	while (fgets(buffer, sizeof(buffer), input) != NULL) {
 	    if (strncasecmp(buffer, "vo", 2) == 0) {
@@ -1709,6 +2063,18 @@ gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
 	    } else if (strncasecmp(buffer, "cachesize", 9) == 0) {
 	    } else if (strncasecmp(buffer, "cache-percent", 13) == 0) {
 	    } else if (strncasecmp(buffer, "dload-dir", 9) == 0) {
+	    } else if (strncasecmp(buffer, "showtime", 8) == 0) {
+	    } else if (strncasecmp(buffer, "enable-wmp", 10) == 0) {
+	    } else if (strncasecmp(buffer, "enable-qt", 9) == 0) {
+	    } else if (strncasecmp(buffer, "enable-rm", 9) == 0) {
+	    } else if (strncasecmp(buffer, "enable-mpeg", 11) == 0) {
+	    } else if (strncasecmp(buffer, "enable-mp3", 10) == 0) {
+	    } else if (strncasecmp(buffer, "enable-ogg", 10) == 0) {
+	    } else if (strncasecmp(buffer, "enable-gmp", 10) == 0) {
+	    } else if (strncasecmp(buffer, "enable-smil", 11) == 0) {
+	    } else if (strncasecmp(buffer, "enable-helix", 12) == 0) {
+	    } else if (strncasecmp(buffer, "nomediacache", 12) == 0) {
+	    } else if (strncasecmp(buffer, "rtsp-use-tcp", 12) == 0) {
 	    } else {
 		fprintf(output, "%s", buffer);
 	    }
@@ -1746,6 +2112,18 @@ gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
 	    fprintf(output, "cache-percent=%i\n", instance->cache_percent);
 	    if (strlen(instance->download_dir) > 0)
 		fprintf(output, "dload-dir=%s\n", instance->download_dir);
+	    fprintf(output, "showtime=%i\n", instance->showtime);
+	    fprintf(output, "enable-wmp=%i\n", instance->enable_wmp);
+	    fprintf(output, "enable-qt=%i\n", instance->enable_qt);
+	    fprintf(output, "enable-rm=%i\n", instance->enable_rm);
+	    fprintf(output, "enable-gmp=%i\n", instance->enable_gmp);
+	    fprintf(output, "enable-mpeg=%i\n", instance->enable_mpeg);
+	    fprintf(output, "enable-mp3=%i\n", instance->enable_mp3);
+	    fprintf(output, "enable-ogg=%i\n", instance->enable_ogg);
+	    fprintf(output, "enable-smil=%i\n", instance->enable_smil);
+	    fprintf(output, "enable-helix=%i\n", instance->enable_helix);
+	    fprintf(output, "nomediacache=%i\n", instance->nomediacache);
+	    fprintf(output, "rtsp-use-tcp=%i\n", instance->rtsp_use_tcp);
 	    fclose(output);
 	    ret = rename(tmp_name, config_name);
 	    if (ret == -1) {
@@ -1759,6 +2137,15 @@ gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
 	}
 
     }
+
+    snprintf(buffer, 1000, "%s", getenv("HOME"));
+    strlcat(buffer, "/.mozilla/pluginreg.dat", 1000);
+    unlink(buffer);
+    snprintf(buffer, 1000, "%s", getenv("HOME"));
+    strlcat(buffer, "/.mozilla/firefox/pluginreg.dat", 1000);
+    unlink(buffer);
+    NPN_ReloadPlugins(FALSE);
+
     return (FALSE);
 }
 
@@ -1771,7 +2158,7 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
     GtkWidget *conf_cancel;
     GtkWidget *conf_table;
     GtkWidget *conf_label;
-
+    int row = 0;
     if (instance->conf_window != NULL)
 	return;
 
@@ -1781,7 +2168,7 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
 
     conf_vbox = gtk_vbox_new(FALSE, 10);
     conf_hbutton_box = gtk_hbutton_box_new();
-    conf_table = gtk_table_new(10, 2, FALSE);
+    conf_table = gtk_table_new(20, 2, FALSE);
 
     gtk_container_add(GTK_CONTAINER(conf_vbox), conf_table);
     gtk_container_add(GTK_CONTAINER(conf_vbox), conf_hbutton_box);
@@ -1793,23 +2180,28 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
     g_signal_connect(GTK_OBJECT(instance->conf_window), "delete_event",
 		     GTK_SIGNAL_FUNC(CloseConfigEvent), instance);
 
+#ifdef GTK24_ENABLED
     instance->conf_vo = gtk_combo_box_entry_new_text();
     if (instance->conf_vo != NULL) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo), "gl");
 	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo), "x11");
 	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo), "xv");
 	if (instance->vo != NULL) {
-	    if (strcmp(instance->vo, "x11") == 0)
+	    if (strcmp(instance->vo, "gl") == 0)
 		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
 					 0);
-	    if (strcmp(instance->vo, "xv") == 0)
+	    if (strcmp(instance->vo, "x11") == 0)
 		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
 					 1);
+	    if (strcmp(instance->vo, "xv") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
+					 2);
 	    if (gtk_combo_box_get_active(GTK_COMBO_BOX(instance->conf_vo))
 		== -1) {
 		gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo),
 					  instance->vo);
 		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
-					 2);
+					 3);
 	    }
 	}
     }
@@ -1863,6 +2255,7 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
 	    }
 	}
     }
+#endif
 
     conf_ok = gtk_button_new_with_label(_("OK"));
     g_signal_connect(GTK_OBJECT(conf_ok), "clicked",
@@ -1870,35 +2263,36 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
 
     conf_cancel = gtk_button_new_with_label(_("Cancel"));
     g_signal_connect(GTK_OBJECT(conf_cancel), "clicked",
-			     GTK_SIGNAL_FUNC(CloseConfig),
-			     instance);
+		     GTK_SIGNAL_FUNC(CloseConfig), instance);
 
+#ifdef GTK24_ENABLED
     conf_label = gtk_label_new(_("Video Output:"));
-    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(conf_label), 0.0, 0.0);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 0,
 			      1);
     gtk_widget_show(conf_label);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), instance->conf_vo, 1,
 			      2, 0, 1);
+
     conf_label = gtk_label_new(_("Audio Output:"));
-    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(conf_label), 0.0, 0.0);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 1,
 			      2);
     gtk_widget_show(conf_label);
-    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(conf_label), 0.0, 0.0);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), instance->conf_ao, 1,
 			      2, 1, 2);
     conf_label = gtk_label_new(_("Save to Location:"));
     gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 2,
 			      3);
     gtk_widget_show(conf_label);
-    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(conf_label), 0.0, 0.0);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), instance->conf_dir, 1,
 			      2, 2, 3);
-
+#endif
 
     conf_label = gtk_label_new(_("Minimum Cache Size:"));
-    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(conf_label), 0.0, 0.0);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 3,
 			      4);
     gtk_widget_show(conf_label);
@@ -1910,7 +2304,7 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
     gtk_widget_show(instance->conf_cachesize);
 
     conf_label = gtk_label_new(_("Percent of Media to Cache:"));
-    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_misc_set_alignment(GTK_MISC(conf_label), 0.0, 0.0);
     gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 4,
 			      5);
     gtk_widget_show(conf_label);
@@ -1921,17 +2315,154 @@ void menuitem_config_callback(GtkMenuItem * menuitem,
 			instance->cache_percent);
     gtk_widget_show(instance->conf_cachepercent);
 
+    row = 5;
+    instance->conf_showtime =
+	gtk_check_button_new_with_label(_("Show time in progress bar"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_showtime, 1, 2, row, row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_showtime),
+				 instance->showtime);
+    gtk_widget_show(instance->conf_showtime);
+
+    instance->conf_enable_wmp =
+	gtk_check_button_new_with_label(_
+					("Enable Windows Media Player Support"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_wmp, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_wmp),
+				 instance->enable_wmp);
+    gtk_widget_show(instance->conf_enable_wmp);
+
+    instance->conf_enable_qt =
+	gtk_check_button_new_with_label(_("Enable QuickTime Support"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_qt, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_qt),
+				 instance->enable_qt);
+    gtk_widget_show(instance->conf_enable_qt);
+
+    instance->conf_enable_rm =
+	gtk_check_button_new_with_label(_("Enable RealMedia Support"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_rm, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_rm),
+				 instance->enable_rm);
+    gtk_widget_show(instance->conf_enable_rm);
+
+    instance->conf_enable_smil =
+	gtk_check_button_new_with_label(_
+					("Enable SMIL Support (Effects QuickTime and RealMedia)"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_smil, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_smil),
+				 instance->enable_smil);
+    gtk_widget_show(instance->conf_enable_smil);
+
+    instance->conf_enable_helix =
+	gtk_check_button_new_with_label(_("Enable Helix Emulation"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_helix, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_helix),
+				 instance->enable_helix);
+    gtk_widget_show(instance->conf_enable_helix);
+
+    instance->conf_enable_gmp =
+	gtk_check_button_new_with_label(_
+					("Enable Google Media Player Support"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_gmp, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_gmp),
+				 instance->enable_gmp);
+    gtk_widget_show(instance->conf_enable_gmp);
+
+    instance->conf_enable_mpeg =
+	gtk_check_button_new_with_label(_("Enable MPEG Support"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_mpeg, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_mpeg),
+				 instance->enable_mpeg);
+    gtk_widget_show(instance->conf_enable_mpeg);
+
+    instance->conf_enable_mp3 =
+	gtk_check_button_new_with_label(_
+					("Enable MP3 Support (Requires MPEG Support)"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_mp3, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_mp3),
+				 instance->enable_mp3);
+    gtk_widget_show(instance->conf_enable_mp3);
+
+    instance->conf_enable_ogg =
+	gtk_check_button_new_with_label(_("Enable Ogg Support"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_enable_ogg, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_enable_ogg),
+				 instance->enable_ogg);
+    gtk_widget_show(instance->conf_enable_ogg);
+
+    instance->conf_nomediacache =
+	gtk_check_button_new_with_label(_
+					("Play media directly from site (No Caching)"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_nomediacache, 1, 2, row,
+			      row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+				 (instance->conf_nomediacache),
+				 instance->nomediacache);
+    gtk_widget_show(instance->conf_nomediacache);
+
+    instance->conf_rtsptcp =
+	gtk_check_button_new_with_label(_
+					("Connect to RTSP Media over TCP"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_rtsptcp, 1, 2, row, row + 1);
+    row++;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(instance->conf_rtsptcp),
+				 instance->rtsp_use_tcp);
+    gtk_widget_show(instance->conf_rtsptcp);
+
+
     gtk_container_add(GTK_CONTAINER(conf_hbutton_box), conf_ok);
     gtk_container_add(GTK_CONTAINER(conf_hbutton_box), conf_cancel);
 
-
+#ifdef GTK24_ENABLED
     if (instance->conf_vo != NULL)
 	gtk_widget_show(instance->conf_vo);
     if (instance->conf_ao != NULL)
 	gtk_widget_show(instance->conf_ao);
     if (instance->conf_dir != NULL)
 	gtk_widget_show(instance->conf_dir);
-
+#endif
     gtk_widget_show(conf_ok);
     gtk_widget_show(conf_cancel);
     gtk_widget_show(conf_table);
@@ -1992,6 +2523,172 @@ gboolean window_visible(GtkWidget * widget, GdkEvent * event,
 
 }
 
-#endif				// GTK2_ENABLED
+gboolean load_href_callback(GtkWidget * widget, GdkEventExpose * event,
+			    nsPluginInstance * instance)
+{
+    Node *n;
+    GdkEventButton *event_button;
+
+    if (DEBUG)
+	printf("image clicked\n");
+
+    if (event->type == GDK_BUTTON_PRESS) {
+	event_button = (GdkEventButton *) event;
+
+	if (event_button->button == 3) {
+	    gtk_menu_popup(instance->popup_menu, NULL, NULL, NULL, NULL,
+			   event_button->button, event_button->time);
+	    return TRUE;
+	}
+
+	if (event_button->button == 1) {
+	    n = instance->list;
+
+	    while (n != NULL) {
+		if (strcmp(n->url, instance->href) == 0)
+		    break;
+		else
+		    n = n->next;
+	    }
+
+	    if (n == NULL) {
+		n = newNode();
+		snprintf(n->url, 1024, "%s", instance->href);
+		n->frombutton = 1;
+		addToEnd(instance->td->list, n);
+		NPN_GetURL(instance->mInstance, instance->href, NULL);
+		if (instance->targetplayer == 0)
+		    gtk_widget_hide(instance->src_event_box);
+	    } else {
+		if (instance->targetplayer == 0)
+		    gtk_widget_hide(instance->src_event_box);
+	    	n->play = 1;
+		instance->Play();
+	    }
+
+	    if (instance->targetplayer == 1)
+		gtk_widget_show(instance->gtkwidget);
+
+	    return TRUE;
+	}
+    }
+
+    return FALSE;
+
+}
+#endif
+
+int srcToButton(char *url, nsPluginInstance * instance)
+{
+#ifndef GTK2_ENABLED
+    return FALSE;
+#endif
+
+#ifdef GTK2_ENABLED
+
+    gchar *command;
+    GError *error = NULL;
+    int exit_status;
+    gchar *dirname;
+    gchar *filename;
+    gboolean result;
+    GtkWidget *fixed_button_container;
+
+    if (DEBUG)
+	printf("In srcToButton\n");
+
+    // setup tmp directory
+    dirname =
+	g_strdup_printf("%s", tempnam("/tmp", "mplayerplug-inXXXXXX"));
+    filename = g_strdup_printf("%s/00000001.jpg", dirname);
+
+    // run mplayer and try to get the first frame and convert it to a jpeg
+    command =
+	g_strdup_printf("mplayer -vo jpeg:outdir=%s -frames 1 %s", dirname,
+			url);
+    if (!g_spawn_command_line_sync
+	(command, NULL, NULL, &exit_status, &error))
+	printf("Error when running When running command: %s\n%s\n",
+	       command, error->message);
+
+    if (fexists(filename)) {
+	instance->pb_src = gdk_pixbuf_new_from_file(filename, &error);
+    } else {
+	instance->pb_src =
+	    gdk_pixbuf_new_from_xpm_data((const char **) start);
+    }
+    if (instance->pb_src != NULL) {
+	if (instance->targetplayer == 0) {
+	    instance->src_event_box = gtk_event_box_new();
+	    instance->image_src =
+		gtk_image_new_from_pixbuf(instance->pb_src);
+	    gtk_container_add(GTK_CONTAINER(instance->src_event_box),
+			      instance->image_src);
+	    g_signal_connect(G_OBJECT(instance->src_event_box),
+			     "button_press_event",
+			     G_CALLBACK(load_href_callback), instance);
+	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
+			  instance->src_event_box, 0, 0);
+	    gtk_widget_show(GTK_WIDGET(instance->image_src));
+	    gtk_widget_show(instance->src_event_box);
+	} else {
+	    instance->button_window = gtk_window_new(GTK_WINDOW_POPUP);
+	    gtk_widget_set_size_request(instance->button_window,
+					instance->window_width,
+					instance->window_height);
+	    gtk_widget_add_events(instance->button_window,
+				  GDK_BUTTON_PRESS_MASK);
+	    gtk_widget_realize(instance->button_window);
+	    instance->src_event_box = gtk_event_box_new();
+	    instance->image_src =
+		gtk_image_new_from_pixbuf(instance->pb_src);
+	    gtk_container_add(GTK_CONTAINER(instance->src_event_box),
+			      instance->image_src);
+	    g_signal_connect(G_OBJECT(instance->src_event_box),
+			     "button_press_event",
+			     G_CALLBACK(load_href_callback), instance);
+	    fixed_button_container = gtk_fixed_new();
+	    gtk_container_add(GTK_CONTAINER(instance->button_window),
+			      fixed_button_container);
+	    gtk_fixed_put(GTK_FIXED(fixed_button_container),
+			  instance->src_event_box, 0, 0);
+	    gtk_widget_show(GTK_WIDGET(instance->image_src));
+	    gtk_widget_show(instance->src_event_box);
+	    gtk_widget_show(fixed_button_container);
+	    gtk_widget_show(instance->button_window);
+	    XReparentWindow(GDK_WINDOW_XDISPLAY
+			    (instance->button_window->window),
+			    GDK_WINDOW_XWINDOW(instance->button_window->
+					       window), instance->window,
+			    0, 0);
+	    gtk_widget_map(instance->button_window);
+	    g_idle_add(gtkgui_draw, instance);
+
+	}
+	result = TRUE;
+    } else {
+	if (instance->targetplayer == 1)
+	    gtk_widget_show(instance->gtkwidget);
+	result = FALSE;
+    }
+    remove(filename);
+    remove(dirname);
+
+    g_free(filename);
+    g_free(dirname);
+    return result;
+
+#endif
+}
+
+gboolean target_hide_callback(GtkWidget * widget, GdkEvent * event,
+			      nsPluginInstance * instance)
+{
+    if (DEBUG)
+	printf("Hiding gtkwidget, window\n");
+    gtk_widget_hide(GTK_WIDGET(instance->gtkwidget));
+    instance->Quit();
+    return TRUE;
+}
 
 #endif				// GTK_ENABLED
