@@ -181,6 +181,7 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     showtracker = 1;
     showbuttons = 1;
     showfsbutton = 1;
+    redrawbuttons = 0;
     mmsstream = 0;
     js_state = JS_STATE_UNDEFINED;
     nQtNext = 0;
@@ -194,16 +195,24 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     panel_height = 0;
     panel_drawn = 0;
     mediaCompleteCallback = NULL;
+    mediaCompleteWithErrorCallback = NULL;
+    mouseClickCallback = NULL;
+    mouseDownCallback = NULL;
+    mouseUpCallback = NULL;
+    mouseEnterCallback = NULL;
+    mouseLeaveCallback = NULL;
     onVisibleCallback = NULL;
     onHiddenCallback = NULL;
     onDestroyCallback = NULL;
     mediaLength = 0.0;
     mediaPercent = 0;
     mediaTime = 0.0;
+    mediaPos = 0.0;
     nomediacache = 0;
     controlsvisible = 0;
     fullscreen = 0;
     showlogo = 1;
+    showtime = 1;
     DPMSEnabled = 0;
     black_background = 0;
     nomouseinput = 1;
@@ -211,6 +220,8 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     cookies = 1;
     starttime = 0;
     nopauseonhide = 0;
+    targetplayer = 0;
+    hidestatus = 0;
 
 #ifdef GTK_ENABLED
     image = NULL;
@@ -223,6 +234,7 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     ff_event_box = NULL;
     rew_event_box = NULL;
     fs_event_box = NULL;
+    src_event_box = NULL;
     gtkwidget = NULL;
     fixed_container = NULL;
     drawing_area = NULL;
@@ -255,6 +267,7 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     novop = 0;
     noembed = 0;
     ao = NULL;
+    af = NULL;
     loop = -1;
     rtsp_use_tcp = 0;
     keep_download = 0;
@@ -269,6 +282,15 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     autosync = 0;
     mc = 0;
     useragent = NULL;
+    enable_smil = 1;
+    enable_helix = 1;
+    enable_wmp = 1;
+    enable_qt = 1;
+    enable_rm = 1;
+    enable_gmp = 1;
+    enable_mpeg = 1;
+    enable_mp3 = 1;
+    enable_ogg = 1;
 
     // JavaScript
     paused = 0;
@@ -277,6 +299,7 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     pthread_mutex_init(&playlist_mutex, NULL);
     pthread_mutex_init(&playlist_cond_mutex, NULL);
     pthread_mutex_init(&control_mutex, NULL);
+    pthread_mutex_init(&read_mutex, NULL);
     pthread_attr_init(&thread_attr);
     pthread_cond_init(&playlist_complete_cond, NULL);
 
@@ -354,16 +377,33 @@ void nsPluginInstance::shutdown()
     }
 
     if (threadsetup == 1) {
-	if (paused == 1)
-	    sendCommand(this, "pause\n");
-	sendCommand(this, "quit\n");
-
-	pthread_cancel(player_thread);
-	js_state = JS_STATE_UNDEFINED;
+	if (threadlaunched == 1) {
+	    pthread_mutex_lock(&control_mutex);
+	    if (paused == 1) {
+		sendCommand(this, "pause\n");
+		paused = 0;
+	    }
+	    sendCommand(this, "quit\n");
+    	    pthread_mutex_lock(&read_mutex);
+	    cancelled = 1;
+    	    pthread_mutex_unlock(&read_mutex);
+	    pthread_mutex_unlock(&control_mutex);
+	    pthread_cancel(player_thread);
+	    pthread_join(player_thread, NULL);
+	    js_state = JS_STATE_UNDEFINED;
+	    
+	}
     }
 
+#ifdef GTK_ENABLED
+    while (g_idle_remove_by_data(this)) {
+	if (DEBUG)
+	    printf("Removing function from idle handler\n");
+    }
+#endif
+
     if (pid != 0) {
-	killmplayer(this);
+        killmplayer(this);
     }
 
     if (DEBUG)
@@ -388,7 +428,12 @@ void nsPluginInstance::shutdown()
 	    if (GTK_IS_WIDGET(conf_window))
 		gtk_widget_destroy(conf_window);
 	}
-
+	if (targetplayer != 0) {
+	    g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
+					delete_signal_id);
+	    if (GTK_IS_WIDGET(button_window))
+		gtk_widget_destroy(button_window);
+	}
 	if (GTK_IS_WIDGET(gtkwidget))
 	    gtk_widget_destroy(gtkwidget);
     }
@@ -399,6 +444,8 @@ void nsPluginInstance::shutdown()
 
 #ifdef GTK1_ENABLED
     if (controlwindow == 0) {
+	if (targetplayer != 0)
+	    gtk_signal_disconnect(GTK_OBJECT(gtkwidget), delete_signal_id);
 	if (GTK_IS_WIDGET(gtkwidget))
 	    gtk_widget_destroy(gtkwidget);
     }
@@ -429,6 +476,11 @@ void nsPluginInstance::shutdown()
     gtkwidget = NULL;
 #endif
 
+#ifdef DPMSExtension
+    if (DPMSEnabled)
+	DPMSReenable(this);
+#endif
+
     if (mimetype != NULL) {
 	free(mimetype);
 	mimetype = NULL;
@@ -520,6 +572,11 @@ void nsPluginInstance::shutdown()
 	mediaCompleteCallback = NULL;
     }
 
+    if (mouseClickCallback != NULL) {
+	NPN_MemFree(mouseClickCallback);
+	mouseClickCallback = NULL;
+    }
+
     if (onVisibleCallback != NULL) {
 	NPN_MemFree(onVisibleCallback);
 	onVisibleCallback = NULL;
@@ -557,6 +614,9 @@ void nsPluginInstance::shutdown()
     pthread_mutex_destroy(&control_mutex);
     if (DEBUG > 1)
 	printf("control_mutex destroyed\n");
+    pthread_mutex_destroy(&read_mutex);
+    if (DEBUG > 1)
+	printf("read_mutex destroyed\n");
     pthread_cond_destroy(&playlist_complete_cond);
     if (DEBUG > 1)
 	printf("playlist_complete_cond destroyed\n");
@@ -626,15 +686,15 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	}
 
 	ws = (NPSetWindowCallbackStruct *) aWindow->ws_info;
+	display = ws->display;
 
+#ifdef X_ENABLED
 	widget =
 	    XtWindowToWidget((Display *) ws->display,
 			     (Window) aWindow->window);
-#ifdef X_ENABLED
 	XtAddEventHandler(widget, ExposureMask, FALSE,
 			  (XtEventHandler) RedrawCB, this);
 #endif
-	display = ws->display;
 #ifdef DPMSExtension
 	DPMSEnabled = DPMSIsEnabled(this);
 #endif
@@ -644,34 +704,53 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	state = STATE_WINDOWSET;
 
 #ifdef GTK_ENABLED
+	gtkplug = gtk_plug_new(window);
+	if (targetplayer == 0) {
+	    gtkwidget = gtk_window_new(GTK_WINDOW_POPUP);
+	} else {
+	    gtkwidget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 #ifdef GTK2_ENABLED
-	gtkwidget = gtk_window_new(GTK_WINDOW_POPUP);
+	    delete_signal_id =
+		g_signal_connect(GTK_OBJECT(gtkwidget), "delete_event",
+				 G_CALLBACK(target_hide_callback), this);
 #endif
 #ifdef GTK1_ENABLED
-	gdk_flush();
-	XSync(display, FALSE);
-	gtkwidget = gtk_plug_new(window);
-	gtk_widget_realize(gtkwidget);
+	    delete_signal_id = gtk_signal_connect(GTK_OBJECT(gtkwidget),
+						  "delete_event",
+						  GTK_SIGNAL_FUNC
+						  (target_hide_callback),
+						  this);
 #endif
-	gdk_flush();
-	gtk_widget_set_events(gtkwidget, GDK_BUTTON_PRESS_MASK);
-	gtk_widget_set_events(gtkwidget, GDK_KEY_PRESS_MASK);
+	    gtk_window_set_title(GTK_WINDOW(gtkwidget),
+				 "mplayerplug-in player");
 
+	}
+	gtk_widget_realize(gtkwidget);
+	gdk_flush();
+	gtk_widget_add_events(gtkwidget, GDK_BUTTON_PRESS_MASK);
+	gtk_widget_add_events(gtkwidget, GDK_BUTTON_RELEASE_MASK);
+	gtk_widget_add_events(gtkwidget, GDK_ENTER_NOTIFY_MASK);
+	gtk_widget_add_events(gtkwidget, GDK_LEAVE_NOTIFY_MASK);
+	gtk_widget_add_events(gtkwidget, GDK_KEY_PRESS_MASK);
+
+	if (targetplayer == 1) {
+	    window_width = 400;
+	    window_height = 200;
+	}
+	if (DEBUG > 1)
+	    printf("setting window size\n");
 #ifdef GTK2_ENABLED
-	gtk_widget_set_size_request(gtkwidget, aWindow->width,
-				    aWindow->height);
-	// logo = gdk_pixbuf_new_from_xpm_data((const char **) logo_xpm);
+	gtk_widget_set_size_request(gtkwidget, window_width,
+				    window_height);
 	logo = gdk_pixbuf_new_from_inline(-1, gtk_logo, FALSE, NULL);
 	image = gtk_image_new_from_pixbuf(logo);
 #endif
-
 #ifdef GTK1_ENABLED
-	if (DEBUG > 1)
-	    printf("setting window size\n");
-	gtk_widget_set_usize(gtkwidget, aWindow->width, aWindow->height);
-
+	gtk_widget_set_usize(gtkwidget, window_width, window_height);
 #endif
 
+
+
 #ifdef HAVE_GETTEXT
 	textdomain("mplayerplug-in");
 	bind_textdomain_codeset("mplayerplug-in", "utf-8");
@@ -731,16 +810,13 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 #ifdef GTK2_ENABLED
 	menuitem_sep3 = GTK_MENU_ITEM(gtk_separator_menu_item_new());
-#endif
-#ifdef GTK1_ENABLED
-	menuitem_sep3 = GTK_MENU_ITEM(gtk_menu_item_new());
-#endif
 	gtk_menu_append(popup_menu, GTK_WIDGET(menuitem_sep3));
 	gtk_widget_show(GTK_WIDGET(menuitem_sep3));
 	menuitem_config =
 	    GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Configure")));
 	gtk_menu_append(popup_menu, GTK_WIDGET(menuitem_config));
 	gtk_widget_show(GTK_WIDGET(menuitem_config));
+#endif
 
 #ifdef GTK2_ENABLED
 	g_signal_connect_swapped(G_OBJECT(gtkwidget),
@@ -765,9 +841,14 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			 G_CALLBACK(menuitem_config_callback), this);
 	// code to try and autopause on window covered up, does not appear GTK sends enough events
 	gtk_widget_add_events(gtkwidget, GDK_VISIBILITY_NOTIFY_MASK);
-	g_signal_connect_after(G_OBJECT(gtkwidget),
-			       "visibility-notify-event",
-			       G_CALLBACK(window_visible), this);
+	g_signal_connect(G_OBJECT(gtkwidget), "button_press_event",
+			 G_CALLBACK(mouse_callback), this);
+	g_signal_connect(G_OBJECT(gtkwidget), "button_release_event",
+			 G_CALLBACK(mouse_callback), this);
+	g_signal_connect(G_OBJECT(gtkwidget), "enter_notify_event",
+			 G_CALLBACK(mousenotify_callback), this);
+	g_signal_connect(G_OBJECT(gtkwidget), "leave_notify_event",
+			 G_CALLBACK(mousenotify_callback), this);
 
 #endif
 
@@ -792,9 +873,8 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			   GTK_SIGNAL_FUNC(menuitem_save_callback), this);
 	gtk_signal_connect(GTK_OBJECT(menuitem_copy), "activate",
 			   GTK_SIGNAL_FUNC(menuitem_copy_callback), this);
-	gtk_signal_connect(GTK_OBJECT(menuitem_config), "activate",
-			   GTK_SIGNAL_FUNC(menuitem_config_callback),
-			   this);
+	gtk_signal_connect(GTK_OBJECT(gtkwidget), "button_press_event",
+			   GTK_SIGNAL_FUNC(mouse_callback), this);
 
 #endif
 	if (DEBUG > 1)
@@ -821,15 +901,15 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	status = GTK_LABEL(gtk_label_new(_("Initializing")));
 	progress_bar = GTK_PROGRESS_BAR(gtk_progress_bar_new());
 
-	if (aWindow->height > 125) {
+	if (window_height > 125) {
 
 #ifdef GTK2_ENABLED
 	    gtk_fixed_put(GTK_FIXED(fixed_container), image, 10, 10);
 	    gtk_widget_set_size_request(GTK_WIDGET(progress_bar),
-					aWindow->width - 20, 15);
+					window_width - 20, 15);
 	    gtk_widget_set_size_request(GTK_WIDGET(status),
-					aWindow->width - 20,
-					aWindow->height - 125);
+					window_width - 20,
+					window_height - 125);
 #endif
 #ifdef GTK1_ENABLED
 	    if (DEBUG > 1)
@@ -848,15 +928,16 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 	    gtk_fixed_put(GTK_FIXED(fixed_container),
 			  GTK_WIDGET(progress_bar), 10,
-			  aWindow->height - 50);
+			  window_height - 50);
 	    if (showlogo)
 		gtk_widget_show(image);
-	    gtk_widget_show(GTK_WIDGET(progress_bar));
+	    if (!hidestatus)
+		gtk_widget_show(GTK_WIDGET(progress_bar));
 	    gtk_fixed_put(GTK_FIXED(fixed_container), GTK_WIDGET(status),
 			  10, 60);
 	} else {
-	    if (aWindow->height < 20) {
-		if (aWindow->width < 126) {
+	    if (window_height < 20) {
+		if (window_width < 126) {
 		    gtk_fixed_put(GTK_FIXED(fixed_container),
 				  GTK_WIDGET(status), 65, 0);
 		} else {
@@ -869,14 +950,15 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	    }
 	    gtk_fixed_put(GTK_FIXED(fixed_container),
 			  GTK_WIDGET(progress_bar), 10,
-			  aWindow->height - 20);
-	    gtk_widget_show(GTK_WIDGET(progress_bar));
+			  window_height - 20);
+	    if (!hidestatus)
+		gtk_widget_show(GTK_WIDGET(progress_bar));
 #ifdef GTK2_ENABLED
 	    gtk_widget_set_size_request(GTK_WIDGET(progress_bar),
-					aWindow->width - 20, 15);
+					window_width - 20, 15);
 	    gtk_widget_set_size_request(GTK_WIDGET(status),
-					aWindow->width - 20,
-					aWindow->height - 25);
+					window_width - 20,
+					window_height - 25);
 #endif
 #ifdef GTK1_ENABLED
 	    if (DEBUG > 1)
@@ -892,10 +974,12 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	gtk_misc_set_alignment(GTK_MISC(status), 0, 0);
 	gtk_label_set_line_wrap(status, TRUE);
 	gtk_container_add(GTK_CONTAINER(gtkwidget), fixed_event_box);
-	gtk_widget_show(GTK_WIDGET(status));
+	if (!hidestatus)
+	    gtk_widget_show(GTK_WIDGET(status));
 	gtk_widget_show(fixed_container);
 	gtk_widget_show(fixed_event_box);
-	gtk_widget_show(gtkwidget);
+	if (targetplayer == 0)
+	    gtk_widget_show(gtkwidget);
 
 	if (black_background != 0) {
 #ifdef GTK_ENABLED
@@ -920,18 +1004,22 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 	gdk_flush();
 	InitPixbufs(this);
+
 	if (DEBUG > 1)
 	    printf("all setup and ready to reparent and map\n");
-#ifdef GTK2_ENABLED
-	XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-			GDK_WINDOW_XWINDOW(gtkwidget->window), window, 0,
-			0);
-	gtk_widget_map(gtkwidget);
+#ifdef GTK_ENABLED
+
+	if (targetplayer == 0) {
+	    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+			    GDK_WINDOW_XWINDOW(gtkwidget->window), window,
+			    0, 0);
+	    gtk_widget_map(gtkwidget);
+	}
 #endif
 
 	if (panel_drawn == 0 && autostart == 0 && nomediacache == 1) {
 	    if (showcontrols) {
-		panel_height = aWindow->height;
+		panel_height = window_height;
 #ifdef GTK_ENABLED
 		g_idle_add(gtkgui_draw, this);
 #endif
@@ -957,21 +1045,11 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 	    if (state != STATE_GETTING_PLAYLIST) {
 		state = STATE_GETTING_PLAYLIST;
-		if (href != NULL) {
-		    if (isMms(href)) {
-			snprintf(list->url, 1024, "%s", href);
-			state = STATE_STARTED_PLAYER;
-			list->mmsstream = 1;
-			if (threadsetup == 0 && controlwindow == 0)
-			    SetupPlayer(this, NULL);
-		    } else {
-			if (autostart)
-			    Play();
-		    }
 
-		}
 		if (url != NULL) {
-		    if (isMms(url)) {
+		    // hard code 0 here by doing this it actually makes nomediacache work properly
+		    // http src url's are automatically requested, so if we request an http here too, we get it twice
+		    if (isMms(url, 0)) {
 			snprintf(list->url, 1024, "%s", url);
 			state = STATE_STARTED_PLAYER;
 			list->mmsstream = 1;
@@ -981,15 +1059,38 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 		}
 
 		if (fname != NULL) {
-		    if (isMms(fname)) {
+		    if (isMms(fname, nomediacache)) {
 			snprintf(list->url, 1024, "%s", fname);
 			state = STATE_STARTED_PLAYER;
 			list->mmsstream = 1;
 			if (threadsetup == 0 && controlwindow == 0)
 			    SetupPlayer(this, NULL);
 		    } else {
-			if (autostart)
+			if (autostart) {
 			    Play();
+			} else {
+			    pthread_mutex_lock(&control_mutex);
+			    js_state = JS_STATE_READY;
+			    pthread_mutex_unlock(&control_mutex);
+			}			
+		    }
+		}
+
+		if ((href != NULL) && (url == NULL) && (fname == NULL)) {
+		    if (isMms(href, nomediacache)) {
+			snprintf(list->url, 1024, "%s", href);
+			state = STATE_STARTED_PLAYER;
+			list->mmsstream = 1;
+			if (threadsetup == 0 && controlwindow == 0)
+			    SetupPlayer(this, NULL);
+		    } else {
+			if (autostart) {
+			    Play();
+			} else {
+			    pthread_mutex_lock(&control_mutex);
+			    js_state = JS_STATE_READY;
+			    pthread_mutex_unlock(&control_mutex);
+			}			
 		    }
 		}
 	    }
@@ -1008,33 +1109,37 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	       aWindow->height);
 
 #ifdef GTK_ENABLED
-    if (status != NULL) {
-	gtk_widget_show(GTK_WIDGET(status));
+    if ((status != NULL) && (targetplayer == 0)) {
+	window_height = aWindow->height;
+	window_width = aWindow->width;
+	if (!hidestatus)
+	    gtk_widget_show(GTK_WIDGET(status));
 #ifdef GTK2_ENABLED
 	gtk_widget_set_size_request(GTK_WIDGET(status),
-				    aWindow->width - 20,
-				    aWindow->height - 25);
+				    window_width - 20, window_height - 25);
 #endif
 #ifdef GTK1_ENABLED
 	gtk_widget_set_usize(GTK_WIDGET(status),
-			     aWindow->width - 20, aWindow->height - 25);
+			     window_width - 20, window_height - 25);
 #endif
     }
     if (gtkwidget == NULL)
 	return NPERR_NO_ERROR;
 
-    gtk_widget_show(gtkwidget);
+    if (targetplayer == 0)
+	gtk_widget_show(gtkwidget);
+/*
     XResizeWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 		  GDK_WINDOW_XWINDOW(gtkwidget->window),
 		  aWindow->width, aWindow->height);
     XResizeWindow(display, window, aWindow->width, aWindow->height);
-
+*/
     if (DEBUG > 1)
 	printf("resize is complete\n");
 
     if (panel_drawn == 0 && autostart == 0 && nomediacache == 1) {
 	if (showcontrols) {
-	    panel_height = aWindow->height;
+	    panel_height = window_height;
 #ifdef GTK_ENABLED
 	    g_idle_add(gtkgui_draw, this);
 #endif
@@ -1174,8 +1279,27 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 		if (n->fname != NULL)
 		    remove(n->fname);
 	    }
+	    printf
+		("checking to see if we need to make a button\nn->url=%s\nurl=%s\nhref=%s\n",
+		 n->url, url, href);
+	    if (n->url != NULL && url != NULL) {
+		if ((strncmp(n->url, url, 1024) == 0) && href != NULL) {
+		    if (srcToButton(n->fname, this)) {
+			n->play = 0;
+			pthread_mutex_unlock(&playlist_mutex);
+			return NPERR_NO_ERROR;
+		    } else {
+			n = newNode();
+			snprintf(n->url, 1024, "%s", href);
+			addToEnd(td->list, n);
+			pthread_mutex_unlock(&playlist_mutex);
+			NPN_GetURL(mInstance, href, NULL);
+			return NPERR_NO_ERROR;
+		    }
+		}
+	    }
 
-	    if (!isMms(n->url)) {
+	    if (!isMms(n->url, nomediacache)) {
 		if (DEBUG)
 		    printf("calling buildPlaylist with filename %s\n",
 			   n->fname);
@@ -1194,7 +1318,9 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 			    gtk_widget_hide(GTK_WIDGET(progress_bar));
 			} else {
 			    if (movie_width == 0 && movie_height == 0)
-				gtk_widget_show(GTK_WIDGET(progress_bar));
+				if (!hidestatus)
+				    gtk_widget_show(GTK_WIDGET
+						    (progress_bar));
 			}
 		    }
 #endif
@@ -1225,6 +1351,10 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 #ifdef GTK_ENABLED
 				    g_idle_add(gtkgui_draw, this);
 #endif
+				    pthread_mutex_lock(&control_mutex);
+				    js_state = JS_STATE_READY;
+				    pthread_mutex_unlock(&control_mutex);
+				
 				}
 			    }
 
@@ -1234,6 +1364,7 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 		}
 	    } else {
 		n->mmsstream = 1;
+		pthread_mutex_unlock(&playlist_mutex);
 	    }
 
 
@@ -1262,6 +1393,10 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 #ifdef GTK_ENABLED
 				g_idle_add(gtkgui_draw, this);
 #endif
+				pthread_mutex_lock(&control_mutex);
+				js_state = JS_STATE_READY;
+				pthread_mutex_unlock(&control_mutex);
+
 			    }
 			}
 		    }
@@ -1281,6 +1416,7 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 
 		    }
 
+
 		    if ((n->bytes <= n->cachebytes) && (n->play == 1)) {
 			all_above_cache = 0;
 			break;
@@ -1295,7 +1431,11 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 			if (autostart) {
 			    signalPlayerThread(this);
 			    threadsignaled = 1;
-			}
+			} else {
+			    pthread_mutex_lock(&control_mutex);
+			    js_state = JS_STATE_READY;
+			    pthread_mutex_unlock(&control_mutex);
+			}			
 		    }
 		}
 	    }
@@ -1318,6 +1458,10 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 #ifdef GTK_ENABLED
 				    g_idle_add(gtkgui_draw, this);
 #endif
+				    pthread_mutex_lock(&control_mutex);
+				    js_state = JS_STATE_READY;
+				    pthread_mutex_unlock(&control_mutex);
+
 				}
 			    }
 			}
@@ -1377,6 +1521,10 @@ void nsPluginInstance::URLNotify(const char *url, NPReason reason,
 #ifdef GTK_ENABLED
 				g_idle_add(gtkgui_draw, this);
 #endif
+				pthread_mutex_lock(&control_mutex);
+				js_state = JS_STATE_READY;
+				pthread_mutex_unlock(&control_mutex);
+			    
 			    }
 			}
 		    }
@@ -1440,7 +1588,9 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 	if (n->retrieved == 1) {
 	    NPN_DestroyStream(mInstance, stream, NPRES_DONE);
 	}
+
 	if ((nomediacache == 1) && (stream->end > 16384)) {
+	    n->mmsstream = 1;
 	    pthread_mutex_unlock(&playlist_mutex);
 	    if (threadsignaled == 0) {
 		if (autostart) {
@@ -1454,11 +1604,14 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 #ifdef GTK_ENABLED
 			g_idle_add(gtkgui_draw, this);
 #endif
+			pthread_mutex_lock(&control_mutex);
+			js_state = JS_STATE_READY;
+			pthread_mutex_unlock(&control_mutex);
+		    
 		    }
 		}
 	    }
 	    return -1;
-
 	} else {
 	    if (strlen(n->fname) == 0) {
 		snprintf(n->fname, 1024, "%s",
@@ -1526,7 +1679,7 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
     int ret = 0;
     long int currdownload, maxdownload;
     FILE *fp;
-    Node *n, *hrefnode;
+    Node *n;
     char message[1024];
     char *burl;
 
@@ -1623,7 +1776,8 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 
 		} else {
 		    if (currdownload < maxdownload) {
-			snprintf(message, 1024, _("Buffering %i%% - %li KB"),
+			snprintf(message, 1024,
+				 _("Buffering %i%% - %li KB"),
 				 (int) (100.0 *
 					((double) currdownload /
 					 (double) maxdownload)),
@@ -1651,14 +1805,17 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 							  1.0) /
 							 (maxdownload *
 							  1.0)));
-				gtk_widget_show(GTK_WIDGET(progress_bar));
+				if (!hidestatus)
+				    gtk_widget_show(GTK_WIDGET
+						    (progress_bar));
 				gtk_widget_queue_draw(GTK_WIDGET
 						      (progress_bar));
 			    }
 			}
 			if (status != NULL) {
 			    gtk_label_set_text(status, message);
-			    gtk_widget_show(GTK_WIDGET(status));
+			    if (!hidestatus)
+				gtk_widget_show(GTK_WIDGET(status));
 			    gtk_widget_queue_draw(GTK_WIDGET(status));
 			}
 #endif
@@ -1680,7 +1837,9 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 							  1.0) /
 							 (maxdownload *
 							  1.0)));
-				gtk_widget_show(GTK_WIDGET(progress_bar));
+				if (!hidestatus)
+				    gtk_widget_show(GTK_WIDGET
+						    (progress_bar));
 				gtk_widget_queue_draw(GTK_WIDGET
 						      (progress_bar));
 			    }
@@ -1688,7 +1847,8 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 			if (status != NULL) {
 			    if (state < STATE_PLAYING) {
 				gtk_label_set_text(status, message);
-				gtk_widget_show(GTK_WIDGET(status));
+				if (!hidestatus)
+				    gtk_widget_show(GTK_WIDGET(status));
 				gtk_widget_queue_draw(GTK_WIDGET(status));
 			    }
 			}
@@ -1696,45 +1856,6 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 		    }
 		}
 	    }
-	    if (href != NULL) {
-		if (hrefrequested == 0) {
-		    if (DEBUG)
-			printf("requesting href\n");
-		    hrefrequested = 1;
-		    n = td->list;
-		    while (n != NULL) {
-			if (URLcmp(n->url, href) == 0) {
-			    break;
-			} else {
-			    if (strstr(stream->url, href) != NULL) {
-				break;
-			    }
-			}
-			n = n->next;
-		    }
-		    if (n == NULL) {
-			if (DEBUG)
-			    printf("href not in list\n");
-			hrefnode = newNode();
-			strlcpy(hrefnode->url, href,
-				sizeof(hrefnode->url));
-			addToEnd(td->list, hrefnode);
-		    } else {
-			if (DEBUG)
-			    printf("href in list\n");
-		    }
-		    NPN_GetURL(mInstance, href, NULL);
-		    pthread_mutex_unlock(&playlist_mutex);
-
-		    if (DEBUG) {
-			printf
-			    ("*******Exiting Write, state = %d, js_state = %d\n",
-			     state, js_state);
-		    }
-
-		    return ret;
-		}
-	    }
 
 	    if (n->status != STATE_DOWNLOADED_ENOUGH) {
 		burl = getURLBase(n->url);
@@ -1751,7 +1872,7 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 		    }
 		}
 
-		if (isMms(n->url)) {
+		if (isMms(n->url, nomediacache)) {
 		    n->mmsstream = 1;
 		}
 	    }
@@ -1763,10 +1884,13 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 				printf("signalling player from write\n");
 			    signalPlayerThread(this);
 			    threadsignaled = 1;
+			} else {
+			    pthread_mutex_lock(&control_mutex);
+			    js_state = JS_STATE_READY;
+			    pthread_mutex_unlock(&control_mutex);
 			}
 		    }
 		} else {
-		    //if (This->mode != NP_FULL) {
 		    if (n->bytes > n->cachebytes) {
 			if (threadsignaled == 0) {
 			    if (autostart) {
@@ -1781,11 +1905,14 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 #ifdef GTK_ENABLED
 				    g_idle_add(gtkgui_draw, this);
 #endif
+				    pthread_mutex_lock(&control_mutex);
+				    js_state = JS_STATE_READY;
+				    pthread_mutex_unlock(&control_mutex);
+				
 				}
 			    }
 			}
 		    }
-		    //}
 		}
 	    }
 
@@ -1818,14 +1945,14 @@ void nsPluginInstance::Play()
     // someone just pressed play so we have to reset the playlist
     if (js_state == JS_STATE_UNDEFINED) {
 	//reset the playlist
-	if (DEBUG > 1)
+	if (DEBUG)
 	    printf("Play: resetting playlist\n");
 
 	pthread_mutex_lock(&playlist_mutex);	// manipulating the playlist, so lock it
 	n = list;
 	while (n != NULL) {
-	    if (n->play)
-		n->played = 0;	// reset playlist
+	    if (n->played == 1)
+		n->played = 0;	// reset played flag
 	    n = n->next;
 	}
 	pthread_mutex_unlock(&playlist_mutex);	// unlock the playlist
@@ -1839,26 +1966,37 @@ void nsPluginInstance::Play()
 	    printf("Play: setupplayer\n");
 	state = STATE_GETTING_PLAYLIST;
 	SetupPlayer(this, NULL);
+	if (nomediacache == 1)
+	    js_state = JS_STATE_BUFFERING;
     }
 
-    if (threadlaunched == 0 && controlwindow == 0) {
+    if (threadsetup == 1 && threadlaunched == 0 && controlwindow == 0) {
 	if (DEBUG > 1)
 	    printf("Play: launching thread\n");
-	pthread_mutex_lock(&control_mutex);	// unlock the control pipe, so that playPlaylist can grab it in a separate thread
+	pthread_mutex_lock(&control_mutex);
 	launchPlayerThread(this);
 	pthread_mutex_unlock(&control_mutex);	// unlock the control pipe, so that playPlaylist can grab it in a separate thread
 
     }
     // we are here cause Play 
     // also the thread has not been signalled to start, so signal it
-    if (threadsignaled == 0 && threadsetup == 1) {
+    if (threadsignaled == 0 && threadsetup == 1 && threadlaunched == 1) {
+	if (DEBUG > 1)
+	    printf("Play: ready to signal\n");
+	while (state < STATE_WAITING_FOR_SIGNAL) {
+	    printf("sleeping state = %i\n", state);
+	    usleep(100);
+	}
+	if (DEBUG > 1)
+	    printf("Play: signalling thread\n");
 	signalPlayerThread(this);
 	threadsignaled = 1;
     }
+
     // normal state, the media is paused or stopped and the thread is active.
     if (paused == 1) {
 	if (DEBUG)
-	    printf("sending play\n");
+	    printf("Play: sending play\n");
 	pthread_mutex_lock(&control_mutex);
 	sendCommand(this, "pause\n");	// send pause again to unpause
 	paused = 0;
@@ -1914,11 +2052,11 @@ void nsPluginInstance::Stop()
     if (mmsstream == 0) {
 	sendCommand(this, "seek 0 2\npause\n");
     } else {
-	pthread_mutex_lock(&playlist_mutex);	// manipulating the playlist, so lock it
+	pthread_mutex_trylock(&playlist_mutex);	// manipulating the playlist, so lock it
 	n = list;
 	while (n != NULL) {
-	    if (n->play)
-		n->played = 1;	// reset playlist
+	    if (n->played)
+		n->played = 0;	// reset playlist
 	    n = n->next;
 	}
 	pthread_mutex_unlock(&playlist_mutex);	// unlock the playlist
@@ -1945,6 +2083,7 @@ void nsPluginInstance::Quit()
     paused = 0;
     js_state = JS_STATE_UNDEFINED;
     pthread_mutex_unlock(&control_mutex);
+    killmplayer(this);
 }
 
 void nsPluginInstance::FastForward()
@@ -2027,6 +2166,9 @@ void nsPluginInstance::GetPlayState(PRInt32 * playstate)
 
 void nsPluginInstance::GetTime(double *_retval)
 {
+    if (js_state == JS_STATE_STOPPED) {
+	mediaTime = 0.0;
+    }
     *_retval = (double) mediaTime;
 }
 
@@ -2105,6 +2247,7 @@ void nsPluginInstance::SetFilename(const char *filename)
 
     deleteList(list);
     list = newNode();
+
     td->list = NULL;
 
     // need to convert to Fully Qualified URL here
@@ -2131,7 +2274,7 @@ void nsPluginInstance::SetFilename(const char *filename)
     cancelled = 0;
     if (DEBUG)
 	printf("SetFilename getting %s\n", localurl);
-    if (!isMms(localurl))
+    if (!isMms(localurl, nomediacache))
 	NPN_GetURL(mInstance, localurl, NULL);
 
 
@@ -2248,6 +2391,11 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	win_width = window_width;
     }
 
+    if (targetplayer == 1) {
+	win_height = movie_height;
+	win_width = movie_width;
+    }
+
     if (DEBUG)
 	printf("height = %i and width = %i\n", win_height, win_width);
 
@@ -2269,18 +2417,20 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	    //gtk_widget_reparent(gtkwidget, fs_window);
 	    gtk_window_get_size(GTK_WINDOW(gtkwidget), &wwidth, &wheight);
 	    if ((wwidth != rect.width) && (wheight != rect.height)) {
-		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-				GDK_WINDOW_XWINDOW(gtkwidget->window),
-				GDK_WINDOW_XWINDOW(fs_window->window), 0,
-				0);
+		if (targetplayer == 0) {
+		    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+				    GDK_WINDOW_XWINDOW(gtkwidget->window),
+				    GDK_WINDOW_XWINDOW(fs_window->window),
+				    0, 0);
 
-		gtk_widget_map(gtkwidget);
+		    gtk_widget_map(gtkwidget);
 
-		gtk_window_resize(GTK_WINDOW(gtkwidget), rect.width,
-				  rect.height);
-		if (DEBUG)
-		    printf("gtkwidget resized to %i x %i\n", rect.width,
-			   rect.height);
+		    gtk_window_resize(GTK_WINDOW(gtkwidget), rect.width,
+				      rect.height);
+		    if (DEBUG)
+			printf("gtkwidget resized to %i x %i\n",
+			       rect.width, rect.height);
+		}
 	    } else {
 		if (DEBUG)
 		    printf("gtkwidget is %i x %i\n", wwidth, wheight);
@@ -2324,16 +2474,22 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 #ifdef GTK2_ENABLED
 	    if (fs_window != NULL) {
 		gtk_window_unfullscreen(GTK_WINDOW(fs_window));
-		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-				GDK_WINDOW_XWINDOW(gtkwidget->window),
-				window, 0, 0);
-		gtk_widget_map(gtkwidget);
+		if (targetplayer == 0) {
+		    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+				    GDK_WINDOW_XWINDOW(gtkwidget->window),
+				    window, 0, 0);
+		    gtk_widget_map(gtkwidget);
+		}
 		gtk_widget_destroy(fs_window);
 		fs_window = NULL;
+	    } else {
+		gtk_window_unfullscreen(GTK_WINDOW(gtkwidget));
+	    }
+	    if (targetplayer == 0) {
+		gtk_window_move(GTK_WINDOW(gtkwidget), 0, 0);
+		gtk_window_resize(GTK_WINDOW(gtkwidget), win_width,
+				  win_height);
 	    }
-	    gtk_window_move(GTK_WINDOW(gtkwidget), 0, 0);
-	    gtk_window_resize(GTK_WINDOW(gtkwidget), win_width,
-			      win_height);
 #endif
 #ifdef GTK1_ENABLED
 	    if (fs_window != NULL) {
@@ -2464,50 +2620,72 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 		printf("SetFullscreen setting fullscreen\n");
 #ifdef GTK_ENABLED
 #ifdef GTK2_ENABLED
-	    fs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	    gtk_widget_set_events(fs_window, GDK_KEY_PRESS_MASK);
-	    gtk_widget_set_events(fs_window, GDK_BUTTON_PRESS_MASK);
+	    if (targetplayer == 0) {
+		fs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_widget_add_events(fs_window, GDK_BUTTON_PRESS_MASK);
+		gtk_widget_add_events(fs_window, GDK_BUTTON_RELEASE_MASK);
+		gtk_widget_add_events(fs_window, GDK_ENTER_NOTIFY_MASK);
+		gtk_widget_add_events(fs_window, GDK_LEAVE_NOTIFY_MASK);
+		gtk_widget_add_events(fs_window, GDK_KEY_PRESS_MASK);
 
-	    g_signal_connect(G_OBJECT(fs_window),
-			     "key_press_event",
-			     G_CALLBACK(keyboard_callback), this);
-	    g_signal_connect_swapped(G_OBJECT(fs_window),
-				     "button_press_event",
-				     G_CALLBACK(popup_handler),
-				     GTK_OBJECT(popup_menu));
+		g_signal_connect(G_OBJECT(fs_window),
+				 "key_press_event",
+				 G_CALLBACK(keyboard_callback), this);
+		g_signal_connect_swapped(G_OBJECT(fs_window),
+					 "button_press_event",
+					 G_CALLBACK(popup_handler),
+					 GTK_OBJECT(popup_menu));
 
-	    screen = gtk_window_get_screen(GTK_WINDOW(gtkwidget));
-	    gtk_window_set_screen(GTK_WINDOW(fs_window), screen);
-	    gtk_window_set_title(GTK_WINDOW(fs_window),
-				 "mplayerplug-in fullscreen");
-	    gdk_screen_get_monitor_geometry(screen,
-					    gdk_screen_get_monitor_at_window
-					    (screen, gtkwidget->window),
-					    &rect);
+		screen = gtk_window_get_screen(GTK_WINDOW(gtkwidget));
+		gtk_window_set_screen(GTK_WINDOW(fs_window), screen);
+		gtk_window_set_title(GTK_WINDOW(fs_window),
+				     "mplayerplug-in fullscreen");
+		gdk_screen_get_monitor_geometry(screen,
+						gdk_screen_get_monitor_at_window
+						(screen,
+						 gtkwidget->window),
+						&rect);
 
-	    x = rect.width;
-	    y = rect.height;
-	    gtk_widget_realize(fs_window);
+		x = rect.width;
+		y = rect.height;
+		gtk_widget_realize(fs_window);
 
-	    gdk_window_get_root_origin(gtkwidget->window, &wx, &wy);
-	    gtk_window_move(GTK_WINDOW(fs_window), wx, wy);
+		gdk_window_get_root_origin(gtkwidget->window, &wx, &wy);
+		gtk_window_move(GTK_WINDOW(fs_window), wx, wy);
 
-	    gtk_widget_show(fs_window);
-	    gtk_window_fullscreen(GTK_WINDOW(fs_window));
-	    gtk_widget_reparent(gtkwidget, fs_window);
-	    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-			    GDK_WINDOW_XWINDOW(gtkwidget->window),
-			    GDK_WINDOW_XWINDOW(fs_window->window), 0, 0);
+		gtk_widget_show(fs_window);
+		gtk_window_fullscreen(GTK_WINDOW(fs_window));
+		gtk_widget_reparent(gtkwidget, fs_window);
+		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+				GDK_WINDOW_XWINDOW(gtkwidget->window),
+				GDK_WINDOW_XWINDOW(fs_window->window), 0,
+				0);
 
-	    gtk_widget_map(gtkwidget);
-	    gtk_window_resize(GTK_WINDOW(gtkwidget), rect.width,
-			      rect.height);
+		gtk_widget_map(gtkwidget);
+		gtk_window_resize(GTK_WINDOW(gtkwidget), rect.width,
+				  rect.height);
+	    } else {
+		screen = gtk_window_get_screen(GTK_WINDOW(gtkwidget));
+		gtk_window_set_title(GTK_WINDOW(gtkwidget),
+				     "mplayerplug-in fullscreen");
+		gdk_screen_get_monitor_geometry(screen,
+						gdk_screen_get_monitor_at_window
+						(screen,
+						 gtkwidget->window),
+						&rect);
 
+		x = rect.width;
+		y = rect.height;
+		gtk_window_fullscreen(GTK_WINDOW(gtkwidget));
+	    }
 #endif
 #ifdef GTK1_ENABLED
 	    fs_window = gtk_window_new(GTK_WINDOW_POPUP);
-	    gtk_widget_set_events(fs_window, GDK_KEY_PRESS_MASK);
-	    gtk_widget_set_events(fs_window, GDK_BUTTON_PRESS_MASK);
+	    gtk_widget_add_events(fs_window, GDK_BUTTON_PRESS_MASK);
+	    gtk_widget_add_events(fs_window, GDK_BUTTON_RELEASE_MASK);
+	    gtk_widget_add_events(fs_window, GDK_ENTER_NOTIFY_MASK);
+	    gtk_widget_add_events(fs_window, GDK_LEAVE_NOTIFY_MASK);
+	    gtk_widget_add_events(fs_window, GDK_KEY_PRESS_MASK);
 	    gtk_widget_realize(fs_window);
 	    gtk_signal_connect(GTK_OBJECT(fs_window),
 			       "key-press-event",
@@ -2624,8 +2802,10 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 #ifdef GTK_ENABLED
 	    gdk_window_set_background(gtkwidget->window, &black);
 	    gdk_window_clear(gtkwidget->window);
-	    gdk_window_set_background(fs_window->window, &black);
-	    gdk_window_clear(fs_window->window);
+	    if (targetplayer == 0) {
+		gdk_window_set_background(fs_window->window, &black);
+		gdk_window_clear(fs_window->window);
+	    }
 	    if (drawing_area != NULL) {
 		gdk_window_set_background(drawing_area->window, &black);
 		gdk_window_clear(drawing_area->window);
@@ -2707,6 +2887,34 @@ void nsPluginInstance::GetMIMEType(char **_retval)
     *_retval = strdup(mimetype);
 }
 
+void nsPluginInstance::PlaylistAppend(const char *item)
+{
+    Node *tail;
+    char localurl[1024];
+
+    pthread_mutex_lock(&playlist_mutex);
+    tail = list;
+    if (tail != NULL) {
+	while (tail->next != NULL)
+	    tail = tail->next;
+    }
+    fullyQualifyURL(this, (char *) item, localurl);
+    addToList(this, localurl, tail, -1);
+    pthread_mutex_unlock(&playlist_mutex);
+}
+
+void nsPluginInstance::PlaylistClear(PRBool * _retval)
+{
+    // can only clear the list when nothing is running
+    if (js_state != JS_STATE_UNDEFINED) {
+	*_retval = (PRBool) FALSE;
+    } else {
+	pthread_mutex_lock(&playlist_mutex);	// manipulating the playlist, so lock it
+	deleteList(list);
+	pthread_mutex_unlock(&playlist_mutex);
+	*_retval = (PRBool) TRUE;
+    }
+}
 
 // ==============================
 // ! Scriptability related code !
@@ -2744,6 +2952,14 @@ NPError nsPluginInstance::GetValue(NPPVariable aVariable, void *aValue)
 	}
 	break;
 
+#ifdef GTK_ENABLED
+    case NPPVpluginNeedsXEmbed:{
+	    *(PRBool *) aValue = PR_TRUE;
+	    rv = NPERR_NO_ERROR;
+	}
+	break;
+#endif
+
     default:
 	break;
     }
