@@ -94,21 +94,24 @@ class nsPluginInstance:public nsPluginInstanceBase {
     void GetTime(double *_retval);
     void GetDuration(double *_retval);
     void GetPercent(double *_retval);
-    void GetFilename(char * *filename);
+    void GetFilename(char **filename);
     void SetFilename(const char *filename);
-    void GetShowControls(PRBool *_retval);
+    void GetShowControls(PRBool * _retval);
     void SetShowControls(PRBool value);
-    void GetFullscreen(PRBool *_retval);
+    void GetFullscreen(PRBool * _retval);
     void SetFullscreen(PRBool value);
-    void GetShowlogo(PRBool *_retval);
+    void GetShowlogo(PRBool * _retval);
     void SetShowlogo(PRBool value);
-    void GetAutoPlay(PRBool *_retval);
+    void GetAutoPlay(PRBool * _retval);
     void SetAutoPlay(PRBool value);
-    void GetLoop(PRBool *_retval);
+    void GetLoop(PRBool * _retval);
     void SetLoop(PRBool value);
     void GetMIMEType(char **_retval);
-    void GetPlaying(PRBool *_retval);
-    
+    void GetPlaying(PRBool * _retval);
+    void PlaylistAppend(const char *item);
+    void PlaylistClear(PRBool *_retval);
+
+
     // we need to provide implementation of this method as it will be
     // used by Mozilla to retrive the scriptable peer
     // and couple of other things on Unix
@@ -116,13 +119,13 @@ class nsPluginInstance:public nsPluginInstanceBase {
 
     nsScriptablePeer *getScriptablePeer();
     nsControlsScriptablePeer *getControlsScriptablePeer();
-    
+
   public:
      NPP mInstance;
     NPBool mInitialized;
     nsScriptablePeer *mScriptablePeer;
     nsControlsScriptablePeer *mControlsScriptablePeer;
-    
+
     // put member data here
     char *mimetype;
     int state;
@@ -145,9 +148,9 @@ class nsPluginInstance:public nsPluginInstanceBase {
     pid_t pid;
     int noredraw;
     int hrefrequested;
-    int threadsetup;			// has the child thread been setup but not signalled
-    int threadlaunched;			// only allow 1 child thread per instance
-    int threadsignaled;			// keep track if the thread has been told to start
+    int threadsetup;		// has the child thread been setup but not signalled
+    int threadlaunched;		// only allow 1 child thread per instance
+    int threadsignaled;		// keep track if the thread has been told to start
     int cancelled;
     int autostart;
     int controlwindow;
@@ -155,6 +158,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int showtracker;
     int showbuttons;
     int showfsbutton;
+    int redrawbuttons;
     int mmsstream;
     Node *list;
     Node *currentnode;
@@ -169,16 +173,24 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int panel_drawn;
     float percent;
     char *mediaCompleteCallback;
+    char *mediaCompleteWithErrorCallback;
+    char *mouseClickCallback;
+    char *mouseDownCallback;
+    char *mouseUpCallback;
+    char *mouseEnterCallback;
+    char *mouseLeaveCallback;
     char *onVisibleCallback;
     char *onHiddenCallback;
     char *onDestroyCallback;
-    float mediaLength;			// length of media in seconds
-    int mediaPercent;			// percentage of media played
-    float mediaTime; 			// time in seconds
+    float mediaLength;		// length of media in seconds
+    int mediaPercent;		// percentage of media played
+    float mediaTime;		// time in seconds
+    float mediaPos;		// seconds into the media
     int nomediacache;
     int controlsvisible;
     int fullscreen;
     int showlogo;
+    int showtime;
     int DPMSEnabled;
     int hidden;
     int black_background;
@@ -187,9 +199,14 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int cookies;
     long int starttime;
     int nopauseonhide;
+    int targetplayer;		// if we have this create a new window and play there
+    int hidestatus;
     
 #ifdef GTK_ENABLED
+    gulong delete_signal_id;
     GtkWidget *gtkwidget;
+    GtkWidget *button_window;
+    GtkWidget *gtkplug;
     GtkWidget *image;
     GtkWidget *fixed_container;
     GtkWidget *fixed_event_box;
@@ -205,14 +222,16 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkWidget *ff_event_box;
     GtkWidget *rew_event_box;
     GtkWidget *fs_event_box;
-    
+    GtkWidget *src_event_box;
+
     GtkWidget *image_play;
     GtkWidget *image_pause;
     GtkWidget *image_stop;
     GtkWidget *image_ff;
     GtkWidget *image_rew;
     GtkWidget *image_fs;
-    
+    GtkWidget *image_src;
+
     GtkMenu *popup_menu;
     GtkMenuItem *menuitem_play;
     GtkMenuItem *menuitem_pause;
@@ -227,12 +246,24 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkMenuItem *menuitem_config;
 
     GtkWidget *conf_window;
-	GtkWidget *conf_vo;
-	GtkWidget *conf_ao;
-	GtkWidget *conf_dir;
-	GtkWidget *conf_cachesize;
-	GtkWidget *conf_cachepercent;
-    
+    GtkWidget *conf_vo;
+    GtkWidget *conf_ao;
+    GtkWidget *conf_dir;
+    GtkWidget *conf_cachesize;
+    GtkWidget *conf_cachepercent;
+    GtkWidget *conf_showtime;
+    GtkWidget *conf_enable_smil;
+    GtkWidget *conf_enable_helix;
+    GtkWidget *conf_nomediacache;
+    GtkWidget *conf_rtsptcp;
+    GtkWidget *conf_enable_wmp;
+    GtkWidget *conf_enable_qt;
+    GtkWidget *conf_enable_rm;
+    GtkWidget *conf_enable_gmp;
+    GtkWidget *conf_enable_ogg;
+    GtkWidget *conf_enable_mpeg;
+    GtkWidget *conf_enable_mp3;
+	
 #endif
 
     pthread_t player_thread;
@@ -241,6 +272,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     pthread_mutex_t playlist_mutex;
     pthread_mutex_t playlist_cond_mutex;
     pthread_mutex_t control_mutex;
+    pthread_mutex_t read_mutex;
 
 #ifdef GTK2_ENABLED
     // Pixbufs - GTK1 does not use Pixbufs
@@ -258,6 +290,8 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GdkPixbuf *pb_sm_rew_down;
     GdkPixbuf *pb_sm_fs_up;
     GdkPixbuf *pb_sm_fs_down;
+    GdkPixbuf *pb_src;
+
     int paused_wheninvisible;
 #endif
 
@@ -306,6 +340,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int novop;
     int noembed;
     char *ao;
+    char *af;
     int loop;
     int rtsp_use_tcp;
     int keep_download;
@@ -323,7 +358,16 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int autosync;
     int mc;
     char *useragent;
-    
+    int enable_smil;
+    int enable_helix;
+    int enable_wmp;
+    int enable_qt;
+    int enable_rm;
+    int enable_gmp;
+    int enable_mpeg;
+    int enable_mp3;
+    int enable_ogg;
+
     // JavaScript State
     int paused;
     int js_state;
